@@ -1,39 +1,11 @@
-/****************************************************************************
-**
-** Copyright (C) 2015 The Qt Company Ltd.
-** Contact: http://www.qt.io/licensing
-**
-** This file is part of Qt Creator.
-**
-** Commercial License Usage
-** Licensees holding valid commercial Qt licenses may use this file in
-** accordance with the commercial license agreement provided with the
-** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and The Qt Company.  For licensing terms and
-** conditions see http://www.qt.io/terms-conditions.  For further information
-** use the contact form at http://www.qt.io/contact-us.
-**
-** GNU Lesser General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU Lesser
-** General Public License version 2.1 or version 3 as published by the Free
-** Software Foundation and appearing in the file LICENSE.LGPLv21 and
-** LICENSE.LGPLv3 included in the packaging of this file.  Please review the
-** following information to ensure the GNU Lesser General Public License
-** requirements will be met: https://www.gnu.org/licenses/lgpl.html and
-** http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
-**
-** In addition, as a special exception, The Qt Company gives you certain additional
-** rights.  These rights are described in The Qt Company LGPL Exception
-** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
-**
-****************************************************************************/
+// Copyright (C) 2016 The Qt Company Ltd.
+// SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only WITH Qt-GPL-exception-1.0
 
 #include "ResolveExpression.h"
 
 #include "LookupContext.h"
 #include "Overview.h"
 #include "DeprecatedGenTemplateInstance.h"
-#include "CppRewriter.h"
 #include "TypeOfExpression.h"
 
 #include <cplusplus/Control.h>
@@ -47,6 +19,8 @@
 #include <cplusplus/NameVisitor.h>
 #include <cplusplus/Templates.h>
 
+#include <utils/algorithm.h>
+
 #include <QList>
 #include <QDebug>
 #include <QSet>
@@ -54,8 +28,9 @@
 #include <map>
 
 using namespace CPlusPlus;
+using namespace Utils;
 
-static const bool debug = ! qgetenv("QTC_LOOKUPCONTEXT_DEBUG").isEmpty();
+static const bool debug = qEnvironmentVariableIsSet("QTC_LOOKUPCONTEXT_DEBUG");
 
 namespace {
 
@@ -64,7 +39,7 @@ static QList<T> removeDuplicates(const QList<T> &results)
 {
     QList<T> uniqueList;
     QSet<T> processed;
-    foreach (const T &r, results) {
+    for (const T &r : results) {
         if (processed.contains(r))
             continue;
 
@@ -88,7 +63,7 @@ public:
         // use an "alreadyResolved" container. FIXME: We might overcome this by resolving the
         // template parameters.
         unsigned maxDepth = 15;
-        for (NamedType *namedTy = 0; maxDepth && (namedTy = getNamedType(*type)); --maxDepth) {
+        for (NamedType *namedTy = nullptr; maxDepth && (namedTy = getNamedType(*type)); --maxDepth) {
             QList<LookupItem> namedTypeItems = getNamedTypeItems(namedTy->name(), *scope, _binding);
 
             if (Q_UNLIKELY(debug))
@@ -130,9 +105,9 @@ private:
         QList<LookupItem> results;
         if (!scope)
             return results;
-        Scope *enclosingBlockScope = 0;
+        Scope *enclosingBlockScope = nullptr;
         for (Block *block = scope->asBlock(); block;
-             block = enclosingBlockScope ? enclosingBlockScope->asBlock() : 0) {
+             block = enclosingBlockScope ? enclosingBlockScope->asBlock() : nullptr) {
             const unsigned memberCount = block->memberCount();
             for (unsigned i = 0; i < memberCount; ++i) {
                 Symbol *symbol = block->memberAt(i);
@@ -165,18 +140,17 @@ private:
                      Scope **scope, QSet<Symbol *>& visited)
     {
         bool foundTypedef = false;
-        foreach (const LookupItem &it, namedTypeItems) {
+        for (const LookupItem &it : namedTypeItems) {
             Symbol *declaration = it.declaration();
             if (declaration && declaration->isTypedef()) {
-                if (visited.contains(declaration))
+                if (!Utils::insert(visited, declaration))
                     break;
-                visited.insert(declaration);
 
                 // continue working with the typedefed type and scope
-                if (type->type()->isPointerType()) {
+                if (type->type()->asPointerType()) {
                     *type = FullySpecifiedType(
                             _context.bindings()->control()->pointerType(declaration->type()));
-                } else if (type->type()->isReferenceType()) {
+                } else if (type->type()->asReferenceType()) {
                     *type = FullySpecifiedType(
                             _context.bindings()->control()->referenceType(
                                 declaration->type(),
@@ -197,7 +171,7 @@ private:
 
     const LookupContext &_context;
     // binding has to be remembered in case of resolving typedefs for templates
-    ClassOrNamespace *_binding;
+    ClassOrNamespace *_binding = nullptr;
 };
 
 static int evaluateFunctionArgument(const FullySpecifiedType &actualTy,
@@ -238,7 +212,7 @@ static int evaluateFunctionArgument(const FullySpecifiedType &actualTy,
 ResolveExpression::ResolveExpression(const LookupContext &context,
                                      const QSet<const Declaration *> &autoDeclarationsBeingResolved)
     : ASTVisitor(context.expressionDocument()->translationUnit()),
-      _scope(0),
+      _scope(nullptr),
       _context(context),
       bind(context.expressionDocument()->translationUnit()),
       _autoDeclarationsBeingResolved(autoDeclarationsBeingResolved),
@@ -257,7 +231,7 @@ QList<LookupItem> ResolveExpression::reference(ExpressionAST *ast, Scope *scope)
 QList<LookupItem> ResolveExpression::resolve(ExpressionAST *ast, Scope *scope, bool ref)
 {
     if (! scope)
-        return QList<LookupItem>();
+        return {};
 
     std::swap(_scope, scope);
     std::swap(_reference, ref);
@@ -286,7 +260,7 @@ QList<LookupItem> ResolveExpression::switchResults(const QList<LookupItem> &resu
 
 void ResolveExpression::addResults(const QList<Symbol *> &symbols)
 {
-    foreach (Symbol *symbol, symbols) {
+    for (Symbol *symbol : symbols) {
         LookupItem item;
         item.setType(symbol->type());
         item.setScope(symbol->enclosingScope());
@@ -319,9 +293,9 @@ bool ResolveExpression::visit(IdExpressionAST *ast)
 
 bool ResolveExpression::visit(BinaryExpressionAST *ast)
 {
-    if (tokenKind(ast->binary_op_token) == T_COMMA && ast->right_expression && ast->right_expression->asQtMethod() != 0) {
+    if (tokenKind(ast->binary_op_token) == T_COMMA && ast->right_expression && ast->right_expression->asQtMethod() != nullptr) {
 
-        if (ast->left_expression && ast->left_expression->asQtMethod() != 0)
+        if (ast->left_expression && ast->left_expression->asQtMethod() != nullptr)
             thisObject();
         else
             accept(ast->left_expression);
@@ -409,7 +383,7 @@ bool ResolveExpression::visit(TypeidExpressionAST *)
 {
     const Name *stdName = control()->identifier("std");
     const Name *tiName = control()->identifier("type_info");
-    const Name *q = control()->qualifiedNameId(control()->qualifiedNameId(/* :: */ 0, stdName), tiName);
+    const Name *q = control()->qualifiedNameId(control()->qualifiedNameId(/* :: */ nullptr, stdName), tiName);
 
     FullySpecifiedType ty(control()->namedType(q));
     addResult(ty, _scope);
@@ -448,7 +422,7 @@ bool ResolveExpression::visit(NumericLiteralAST *ast)
 {
     const Token &tk = tokenAt(ast->literal_token);
 
-    Type *type = 0;
+    Type *type = nullptr;
     bool isUnsigned = false;
 
     if (tk.is(T_CHAR_LITERAL)) {
@@ -586,21 +560,17 @@ bool ResolveExpression::visit(UnaryExpressionAST *ast)
     accept(ast->expression);
     unsigned unaryOp = tokenKind(ast->unary_op_token);
     if (unaryOp == T_AMPER) {
-        QMutableListIterator<LookupItem > it(_results);
-        while (it.hasNext()) {
-            LookupItem p = it.next();
+        for (LookupItem &p : _results) {
             FullySpecifiedType ty = p.type();
             ty.setType(control()->pointerType(ty));
             p.setType(ty);
-            it.setValue(p);
         }
     } else if (unaryOp == T_STAR) {
-        QMutableListIterator<LookupItem > it(_results);
-        while (it.hasNext()) {
-            LookupItem p = it.next();
+        for (int i = 0; i < _results.size(); ++i) {
+            LookupItem &p = _results[i];
             FullySpecifiedType ty = p.type();
             NamedType *namedTy = ty->asNamedType();
-            if (namedTy != 0) {
+            if (namedTy != nullptr) {
                 const QList<LookupItem> types = _context.lookup(namedTy->name(), p.scope());
                 if (!types.empty())
                     ty = types.front().type();
@@ -608,12 +578,12 @@ bool ResolveExpression::visit(UnaryExpressionAST *ast)
             bool added = false;
             if (PointerType *ptrTy = ty->asPointerType()) {
                 p.setType(ptrTy->elementType());
-                it.setValue(p);
                 added = true;
-            } else if (namedTy != 0) {
+            } else if (namedTy != nullptr) {
                 const Name *starOp = control()->operatorNameId(OperatorNameId::StarOp);
-                if (ClassOrNamespace *b = _context.lookupType(namedTy->name(), p.scope())) {
-                    foreach (const LookupItem &r, b->find(starOp)) {
+                if (ClassOrNamespace *b = _context.lookupType(namedTy->name(), p.scope(), p.binding())) {
+                    const QList<LookupItem> results = b->find(starOp);
+                    for (const LookupItem &r : results) {
                         Symbol *overload = r.declaration();
                         if (Function *funTy = overload->type()->asFunctionType()) {
                             if (maybeValidPrototype(funTy, 0)) {
@@ -621,7 +591,6 @@ bool ResolveExpression::visit(UnaryExpressionAST *ast)
                                     FullySpecifiedType retTy = proto->returnType().simplified();
                                     p.setType(retTy);
                                     p.setScope(proto->enclosingScope());
-                                    it.setValue(p);
                                     added = true;
                                     break;
                                 }
@@ -631,7 +600,7 @@ bool ResolveExpression::visit(UnaryExpressionAST *ast)
                 }
             }
             if (!added)
-                it.remove();
+                _results.removeAt(i--);
         }
     }
     return false;
@@ -664,7 +633,7 @@ public:
         accept(tu->ast());
     }
 
-    virtual bool preVisit(AST *)
+    bool preVisit(AST *) override
     {
         if (_block)
             return false;
@@ -672,7 +641,7 @@ public:
         return true;
     }
 
-    virtual bool visit(SimpleNameAST *ast)
+    bool visit(SimpleNameAST *ast) override
     {
         if (ast->name
                 && ast->name->identifier()
@@ -683,7 +652,7 @@ public:
         return false;
     }
 
-    virtual bool visit(MemberAccessAST *ast)
+    bool visit(MemberAccessAST *ast) override
     {
         accept(ast->base_expression);
         return false;
@@ -693,6 +662,31 @@ public:
     bool _block;
 };
 
+class ExpressionDocumentHelper
+{
+public:
+    // Set up an expression document with an external Control
+    ExpressionDocumentHelper(const QByteArray &utf8code, Control *control)
+        : document(Document::create(FilePath::fromPathPart(u"<completion>")))
+    {
+        Control *oldControl = document->swapControl(control);
+        delete oldControl->diagnosticClient();
+        delete oldControl;
+        document->setUtf8Source(utf8code);
+        document->parse(Document::ParseExpression);
+        document->check();
+    }
+
+    // Ensure that the external Control is not deleted
+    ~ExpressionDocumentHelper()
+    {
+        document->swapControl(nullptr);
+    }
+
+public:
+    Document::Ptr document;
+};
+
 } // namespace anonymous
 
 bool ResolveExpression::visit(SimpleNameAST *ast)
@@ -705,7 +699,7 @@ bool ResolveExpression::visit(SimpleNameAST *ast)
         if (!item.type()->isUndefinedType())
             continue;
 
-        if (item.declaration() == 0)
+        if (item.declaration() == nullptr)
             continue;
 
         if (item.type().isAuto()) {
@@ -718,7 +712,7 @@ bool ResolveExpression::visit(SimpleNameAST *ast)
                 continue;
 
             const StringLiteral *initializationString = decl->getInitializer();
-            if (initializationString == 0)
+            if (initializationString == nullptr)
                 continue;
 
             const QByteArray &initializer =
@@ -730,13 +724,14 @@ bool ResolveExpression::visit(SimpleNameAST *ast)
                 continue;
 
             TypeOfExpression exprTyper;
-            Document::Ptr doc = _context.snapshot().document(QString::fromLocal8Bit(decl->fileName()));
+            exprTyper.setExpandTemplates(true);
+            Document::Ptr doc = _context.snapshot().document(decl->filePath());
             exprTyper.init(doc, _context.snapshot(), _context.bindings(),
                            QSet<const Declaration* >(_autoDeclarationsBeingResolved) << decl);
 
-            Document::Ptr exprDoc =
-                    documentForExpression(exprTyper.preprocessedExpression(initializer));
-            exprDoc->check();
+            const ExpressionDocumentHelper exprHelper(exprTyper.preprocessedExpression(initializer),
+                                                      _context.bindings()->control().data());
+            const Document::Ptr exprDoc = exprHelper.document;
 
             DeduceAutoCheck deduceAuto(ast->name->identifier(), exprDoc->translationUnit());
             if (deduceAuto._block)
@@ -747,17 +742,19 @@ bool ResolveExpression::visit(SimpleNameAST *ast)
             if (typeItems.empty())
                 continue;
 
-            CPlusPlus::Clone cloner(_context.bindings()->control().data());
+            Clone cloner(_context.bindings()->control().data());
 
             for (int n = 0; n < typeItems.size(); ++ n) {
-                FullySpecifiedType newType = cloner.type(typeItems[n].type(), 0);
+                FullySpecifiedType newType = cloner.type(typeItems[n].type(), nullptr);
                 if (n == 0) {
                     item.setType(newType);
                     item.setScope(typeItems[n].scope());
+                    item.setBinding(typeItems[n].binding());
                 } else {
                     LookupItem newItem(item);
                     newItem.setType(newType);
                     newItem.setScope(typeItems[n].scope());
+                    newItem.setBinding(typeItems[n].binding());
                     newCandidates.push_back(newItem);
                 }
             }
@@ -824,7 +821,7 @@ bool ResolveExpression::visit(CallAST *ast)
     if (_reference) {
         typedef std::multimap<int, LookupItem> LookupMap;
         LookupMap sortedResults;
-        foreach (const LookupItem &base, baseResults) {
+        for (const LookupItem &base : baseResults) {
             if (Function *funTy = base.type()->asFunctionType()) {
                 if (! maybeValidPrototype(funTy, actualArgumentCount))
                     continue;
@@ -832,22 +829,14 @@ bool ResolveExpression::visit(CallAST *ast)
                 int score = 0;
 
                 for (unsigned i = 0, argc = funTy->argumentCount(); i < argc; ++i) {
+                    if (i >= unsigned(arguments.size()))
+                        break;
+
                     const FullySpecifiedType formalTy = funTy->argumentAt(i)->type();
-
-                    FullySpecifiedType actualTy;
-                    if (i < unsigned(arguments.size())) {
-                        const QList<LookupItem> actual = arguments.at(i);
-                        if (actual.isEmpty())
-                            continue;
-
-                        actualTy = actual.first().type();
-                    } else {
-                        actualTy = formalTy;
-                        score += 2;
+                    const QList<LookupItem> actual = arguments.at(i);
+                    if (actual.isEmpty())
                         continue;
-                    }
-
-                    score += evaluateFunctionArgument(actualTy, formalTy);
+                    score += evaluateFunctionArgument(actual.first().type(), formalTy);
                 }
 
                 sortedResults.insert(LookupMap::value_type(-score, base));
@@ -865,18 +854,25 @@ bool ResolveExpression::visit(CallAST *ast)
 
     const Name *functionCallOp = control()->operatorNameId(OperatorNameId::FunctionCallOp);
 
-    foreach (const LookupItem &result, baseResults) {
+    for (const LookupItem &result : baseResults) {
         FullySpecifiedType ty = result.type().simplified();
         Scope *scope = result.scope();
 
         if (NamedType *namedTy = ty->asNamedType()) {
             if (ClassOrNamespace *b = _context.lookupType(namedTy->name(), scope)) {
-                foreach (const LookupItem &r, b->find(functionCallOp)) {
-                    Symbol *overload = r.declaration();
-                    if (Function *funTy = overload->type()->asFunctionType()) {
-                        if (maybeValidPrototype(funTy, actualArgumentCount)) {
-                            if (Function *proto = instantiate(namedTy->name(), funTy)->asFunctionType())
-                                addResult(proto->returnType().simplified(), scope);
+                if (b->templateId() && result.declaration() && result.declaration()->asTemplate()) {
+                    // Template class constructor
+                    addResult(ty.simplified(), scope);
+                } else {
+                    // operator()
+                    const QList<LookupItem> results = b->find(functionCallOp);
+                    for (const LookupItem &r : results) {
+                        Symbol *overload = r.declaration();
+                        if (Function *funTy = overload->type()->asFunctionType()) {
+                            if (maybeValidPrototype(funTy, actualArgumentCount)) {
+                                if (Function *proto = instantiate(namedTy->name(), funTy)->asFunctionType())
+                                    addResult(proto->returnType().simplified(), scope);
+                            }
                         }
                     }
                 }
@@ -901,6 +897,13 @@ bool ResolveExpression::visit(CallAST *ast)
         }
     }
 
+    if (_results.size()>1){
+        // move functions with known bindings to begin of results list
+        std::stable_partition(_results.begin(), _results.end(), [](const LookupItem &item) -> bool {
+            return item.binding();
+        });
+    }
+
     return false;
 }
 
@@ -909,7 +912,7 @@ bool ResolveExpression::visit(ArrayAccessAST *ast)
     const QList<LookupItem> baseResults = resolve(ast->base_expression, _scope);
     const Name *arrayAccessOp = control()->operatorNameId(OperatorNameId::ArrayAccessOp);
 
-    foreach (const LookupItem &result, baseResults) {
+    for (const LookupItem &result : baseResults) {
         FullySpecifiedType ty = result.type().simplified();
         Scope *scope = result.scope();
 
@@ -924,12 +927,12 @@ bool ResolveExpression::visit(ArrayAccessAST *ast)
 
         } else if (NamedType *namedTy = ty->asNamedType()) {
             if (ClassOrNamespace *b = _context.lookupType(namedTy->name(), scope)) {
-                foreach (const LookupItem &r, b->find(arrayAccessOp)) {
+                const QList<LookupItem> results = b->find(arrayAccessOp);
+                for (const LookupItem &r : results) {
                     Symbol *overload = r.declaration();
                     if (Function *funTy = overload->type()->asFunctionType()) {
-                        if (Function *proto = instantiate(namedTy->name(), funTy)->asFunctionType())
-                            // ### TODO: check the actual arguments
-                            addResult(proto->returnType().simplified(), scope);
+                        // ### TODO: check the actual arguments
+                        addResult(funTy->returnType().simplified(), scope, b);
                     }
                 }
 
@@ -941,15 +944,15 @@ bool ResolveExpression::visit(ArrayAccessAST *ast)
 
 QList<LookupItem> ResolveExpression::getMembers(ClassOrNamespace *binding, const Name *memberName) const
 {
-    Q_UNUSED(binding);
-    Q_UNUSED(memberName);
+    Q_UNUSED(binding)
+    Q_UNUSED(memberName)
 
     // ### port me
     QList<LookupItem> members;
 #if 0
     const QList<LookupItem> originalMembers = binding->find(memberName);
 
-    foreach (const LookupItem &m, originalMembers) {
+    for (const LookupItem &m, originalMembers) {
         if (! m.binding() || ! m.binding()->templateId()) {
             members.append(m);
             continue;
@@ -1001,7 +1004,7 @@ bool ResolveExpression::visit(MemberAccessAST *ast)
     const QList<LookupItem> baseResults = resolve(ast->base_expression, _scope);
 
     // Evaluate the expression-id that follows the access operator.
-    const Name *memberName = 0;
+    const Name *memberName = nullptr;
     if (ast->member_name)
         memberName = ast->member_name->name;
 
@@ -1015,20 +1018,20 @@ bool ResolveExpression::visit(MemberAccessAST *ast)
 }
 
 ClassOrNamespace *ResolveExpression::findClass(const FullySpecifiedType &originalTy, Scope *scope,
-                                               ClassOrNamespace* enclosingTemplateInstantiation) const
+                                               ClassOrNamespace *enclosingBinding) const
 {
     FullySpecifiedType ty = originalTy.simplified();
-    ClassOrNamespace *binding = 0;
+    ClassOrNamespace *binding = nullptr;
 
     if (Class *klass = ty->asClassType()) {
-        if (scope->isBlock())
-            binding = _context.lookupType(klass->name(), scope, enclosingTemplateInstantiation);
+        if (scope->asBlock())
+            binding = _context.lookupType(klass->name(), scope, enclosingBinding);
         if (!binding)
-            binding = _context.lookupType(klass, enclosingTemplateInstantiation);
+            binding = _context.lookupType(klass, enclosingBinding);
     }
 
     else if (NamedType *namedTy = ty->asNamedType())
-        binding = _context.lookupType(namedTy->name(), scope, enclosingTemplateInstantiation);
+        binding = _context.lookupType(namedTy->name(), scope, enclosingBinding);
 
     else if (Function *funTy = ty->asFunctionType())
         return findClass(funTy->returnType(), scope);
@@ -1046,7 +1049,7 @@ ClassOrNamespace *ResolveExpression::baseExpression(const QList<LookupItem> &bas
     Overview oo;
     TypedefsResolver typedefsResolver(_context);
 
-    foreach (const LookupItem &r, baseResults) {
+    for (const LookupItem &r : baseResults) {
         if (!r.type().type() || !r.scope())
             continue;
         FullySpecifiedType ty = r.type().simplified();
@@ -1055,16 +1058,21 @@ ClassOrNamespace *ResolveExpression::baseExpression(const QList<LookupItem> &bas
 
         if (Q_UNLIKELY(debug)) {
             qDebug("trying result #%d", ++i);
-            qDebug() << "- before typedef resolving we have:" << oo(ty);
+            qDebug() << "- before typedef resolving we have:" << oo.prettyType(ty);
         }
 
         typedefsResolver.resolve(&ty, &scope, r.binding());
 
         if (Q_UNLIKELY(debug))
-            qDebug() << "-  after typedef resolving:" << oo(ty);
+            qDebug() << "-  after typedef resolving:" << oo.prettyType(ty);
 
         if (accessOp == T_ARROW) {
-            if (PointerType *ptrTy = ty->asPointerType()) {
+            PointerType *ptrTy = ty->asPointerType();
+            if (!ptrTy) {
+                if (Function * const func = ty->asFunctionType())
+                    ptrTy = func->returnType()->asPointerType();
+            }
+            if (ptrTy) {
                 FullySpecifiedType type = ptrTy->elementType();
                 if (ClassOrNamespace *binding
                         = findClassForTemplateParameterInExpressionScope(r.binding(),
@@ -1078,32 +1086,46 @@ ClassOrNamespace *ResolveExpression::baseExpression(const QList<LookupItem> &bas
                 ClassOrNamespace *binding
                         = findClassForTemplateParameterInExpressionScope(r.binding(),
                                                                          ty);
+
                 if (! binding)
-                    binding = findClass(ty, scope);
+                    binding = findClass(ty, scope, r.binding());
 
                 if (binding){
                     // lookup for overloads of operator->
 
                     const OperatorNameId *arrowOp
                             = control()->operatorNameId(OperatorNameId::ArrowOp);
-                    foreach (const LookupItem &r, binding->find(arrowOp)) {
+                    const QList<LookupItem> results = binding->find(arrowOp);
+                    for (const LookupItem &r : results) {
                         Symbol *overload = r.declaration();
                         if (! overload)
                             continue;
                         Scope *functionScope = overload->enclosingScope();
 
-                        if (overload->type()->isFunctionType()) {
+                        FullySpecifiedType overloadType = r.type();
+                        if (! overloadType.isValid())
+                            overloadType = overload->type();
+
+                        Function *instantiatedFunction = nullptr;
+
+                        if (overloadType->asFunctionType()) {
                             FullySpecifiedType overloadTy
                                     = instantiate(binding->templateId(), overload);
-                            Function *instantiatedFunction = overloadTy->asFunctionType();
-                            Q_ASSERT(instantiatedFunction != 0);
+                            instantiatedFunction = overloadTy->asFunctionType();
+                        } else if (overloadType->asTemplateType()
+                                   && overloadType->asTemplateType()->declaration()
+                                   && overloadType->asTemplateType()->declaration()->asFunction()) {
+                            instantiatedFunction = overloadType->asTemplateType()->declaration()->asFunction();
+                        }
+
+                        if (instantiatedFunction != nullptr) {
 
                             FullySpecifiedType retTy
                                     = instantiatedFunction->returnType().simplified();
 
                             typedefsResolver.resolve(&retTy, &functionScope, r.binding());
 
-                            if (! retTy->isPointerType() && ! retTy->isNamedType())
+                            if (! retTy->asPointerType() && ! retTy->asNamedType())
                                 continue;
 
                             if (PointerType *ptrTy = retTy->asPointerType())
@@ -1118,7 +1140,8 @@ ClassOrNamespace *ResolveExpression::baseExpression(const QList<LookupItem> &bas
                             }
 
                             if (ClassOrNamespace *origin = binding->instantiationOrigin()) {
-                                foreach (Symbol *originSymbol, origin->symbols()) {
+                                const QList<Symbol *> symbols = origin->symbols();
+                                for (Symbol *originSymbol : symbols) {
                                     Scope *originScope = originSymbol->asScope();
                                     if (originScope && originScope != scope
                                             && originScope != functionScope) {
@@ -1134,7 +1157,7 @@ ClassOrNamespace *ResolveExpression::baseExpression(const QList<LookupItem> &bas
             }
         } else if (accessOp == T_DOT) {
             if (replacedDotOperator) {
-                *replacedDotOperator = originalType->isPointerType() || ty->isPointerType();
+                *replacedDotOperator = originalType->asPointerType() || ty->asPointerType();
                 if (PointerType *ptrTy = ty->asPointerType())
                     ty = ptrTy->elementType();
             }
@@ -1145,18 +1168,18 @@ ClassOrNamespace *ResolveExpression::baseExpression(const QList<LookupItem> &bas
                 return binding;
             }
 
-            ClassOrNamespace *enclosingTemplateInstantiation = 0;
+            ClassOrNamespace *enclosingBinding = nullptr;
             if (ClassOrNamespace *binding = r.binding()) {
                 if (binding->instantiationOrigin())
-                    enclosingTemplateInstantiation = binding;
+                    enclosingBinding = binding;
             }
 
-            if (ClassOrNamespace *binding = findClass(ty, scope, enclosingTemplateInstantiation))
+            if (ClassOrNamespace *binding = findClass(ty, scope, enclosingBinding))
                 return binding;
         }
     }
 
-    return 0;
+    return nullptr;
 }
 
 ClassOrNamespace *ResolveExpression::findClassForTemplateParameterInExpressionScope(
@@ -1165,7 +1188,8 @@ ClassOrNamespace *ResolveExpression::findClassForTemplateParameterInExpressionSc
 {
     if (resultBinding) {
         if (ClassOrNamespace *origin = resultBinding->instantiationOrigin()) {
-            foreach (Symbol *originSymbol, origin->symbols()) {
+            const QList<Symbol *> symbols = origin->symbols();
+            for (Symbol *originSymbol : symbols) {
                 if (Scope *originScope = originSymbol->asScope()) {
                     if (ClassOrNamespace *retBinding = findClass(ty, originScope))
                         return retBinding;
@@ -1174,7 +1198,7 @@ ClassOrNamespace *ResolveExpression::findClassForTemplateParameterInExpressionSc
         }
     }
 
-    return 0;
+    return nullptr;
 }
 
 FullySpecifiedType ResolveExpression::instantiate(const Name *className, Symbol *candidate) const
@@ -1194,9 +1218,9 @@ bool ResolveExpression::visit(ObjCMessageExpressionAST *ast)
 {
     const QList<LookupItem> receiverResults = resolve(ast->receiver_expression, _scope);
 
-    foreach (const LookupItem &result, receiverResults) {
+    for (const LookupItem &result : receiverResults) {
         FullySpecifiedType ty = result.type().simplified();
-        ClassOrNamespace *binding = 0;
+        ClassOrNamespace *binding = nullptr;
 
         if (ObjCClass *clazz = ty->asObjCClassType()) {
             // static access, e.g.:
@@ -1211,7 +1235,8 @@ bool ResolveExpression::visit(ObjCMessageExpressionAST *ast)
         }
 
         if (binding) {
-            foreach (const LookupItem &r, binding->lookup(ast->selector->name)) {
+            const QList<LookupItem> results = binding->lookup(ast->selector->name);
+            for (const LookupItem &r : results) {
                 Symbol *s = r.declaration();
                 if (ObjCMethod *m = s->asObjCMethod())
                     addResult(m->returnType(), result.scope());
