@@ -1,39 +1,23 @@
-/****************************************************************************
-**
-** Copyright (C) 2016 The Qt Company Ltd.
-** Contact: https://www.qt.io/licensing/
-**
-** This file is part of Qt Creator.
-**
-** Commercial License Usage
-** Licensees holding valid commercial Qt licenses may use this file in
-** accordance with the commercial license agreement provided with the
-** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and The Qt Company. For licensing terms
-** and conditions see https://www.qt.io/terms-conditions. For further
-** information use the contact form at https://www.qt.io/contact-us.
-**
-** GNU General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU
-** General Public License version 3 as published by the Free Software
-** Foundation with exceptions as appearing in the file LICENSE.GPL3-EXCEPT
-** included in the packaging of this file. Please review the following
-** information to ensure the GNU General Public License requirements will
-** be met: https://www.gnu.org/licenses/gpl-3.0.html.
-**
-****************************************************************************/
+// Copyright (C) 2016 The Qt Company Ltd.
+// SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only WITH Qt-GPL-exception-1.0
 
 #include "pathchooser.h"
 
+#include "async.h"
 #include "commandline.h"
 #include "environment.h"
 #include "fileutils.h"
+#include "guard.h"
 #include "hostosinfo.h"
 #include "macroexpander.h"
+#include "optionpushbutton.h"
+#include "process.h"
 #include "qtcassert.h"
-#include "qtcprocess.h"
+#include "utilstr.h"
 
 #include <QFileDialog>
+#include <QFuture>
+#include <QGuiApplication>
 #include <QHBoxLayout>
 #include <QMenu>
 #include <QPushButton>
@@ -148,7 +132,7 @@ QString BinaryVersionToolTipEventFilter::toolVersion(const CommandLine &cmd)
 {
     if (cmd.executable().isEmpty())
         return QString();
-    QtcProcess proc;
+    Process proc;
     proc.setTimeoutS(1);
     proc.setCommand(cmd);
     proc.runBlocking();
@@ -189,11 +173,16 @@ public:
     FilePath m_initialBrowsePathOverride;
     QString m_defaultValue;
     FilePath m_baseDirectory;
-    EnvironmentChange m_environmentChange;
+    Environment m_environment;
     BinaryVersionToolTipEventFilter *m_binaryVersionToolTipEventFilter = nullptr;
     QList<QAbstractButton *> m_buttons;
     const MacroExpander *m_macroExpander = globalMacroExpander();
     std::function<void()> m_openTerminal;
+    bool m_allowPathFromDevice = false;
+
+    QMenu *m_contextMenu = nullptr;
+    OptionPushButton *m_browseButton = nullptr;
+    Guard m_callGuard;
 };
 
 PathChooserPrivate::PathChooserPrivate()
@@ -209,8 +198,7 @@ FilePath PathChooserPrivate::expandedPath(const FilePath &input) const
 
     FilePath path = input;
 
-    Environment env = path.deviceEnvironment();
-    m_environmentChange.applyToEnvironment(env);
+    Environment env = m_environment.appliedToEnvironment(path.deviceEnvironment());
     path = env.expandVariables(path);
 
     if (m_macroExpander)
@@ -219,6 +207,9 @@ FilePath PathChooserPrivate::expandedPath(const FilePath &input) const
     if (path.isEmpty())
         return path;
 
+    if (path.isAbsolutePath())
+        return path;
+
     switch (m_acceptingKind) {
     case PathChooser::Command:
     case PathChooser::ExistingCommand: {
@@ -262,21 +253,24 @@ PathChooser::PathChooser(QWidget *parent) :
             this,
             &PathChooser::contextMenuRequested);
     connect(d->m_lineEdit, &FancyLineEdit::validReturnPressed, this, &PathChooser::returnPressed);
-    connect(d->m_lineEdit, &QLineEdit::textChanged, this,
-            [this] { emit rawPathChanged(rawPath()); });
+    connect(d->m_lineEdit, &QLineEdit::textChanged, this, &PathChooser::rawPathChanged);
     connect(d->m_lineEdit, &FancyLineEdit::validChanged, this, &PathChooser::validChanged);
     connect(d->m_lineEdit, &QLineEdit::editingFinished, this, &PathChooser::editingFinished);
-    connect(d->m_lineEdit, &QLineEdit::textChanged, this, [this] {
-        const QString text = d->m_lineEdit->text();
-        emit pathChanged(text);
-        emit filePathChanged(FilePath::fromUserInput(text));
-    });
+    connect(d->m_lineEdit, &QLineEdit::textChanged, this, &PathChooser::textChanged);
 
     d->m_lineEdit->setMinimumWidth(120);
     d->m_hLayout->addWidget(d->m_lineEdit);
     d->m_hLayout->setSizeConstraint(QLayout::SetMinimumSize);
 
-    addButton(browseButtonLabel(), this, [this] { slotBrowse(); });
+    d->m_browseButton = new OptionPushButton;
+    d->m_browseButton->setText(browseButtonLabel());
+    connect(d->m_browseButton, &OptionPushButton::clicked, this, [this] { slotBrowse(false); });
+
+    d->m_contextMenu = new QMenu(d->m_browseButton);
+    d->m_contextMenu->addAction(Tr::tr("Local"), this, [this] { slotBrowse(false); });
+    d->m_contextMenu->addAction(Tr::tr("Remote"), this, [this] { slotBrowse(true); });
+
+    insertButton(d->m_buttons.count(), d->m_browseButton);
 
     setLayout(d->m_hLayout);
     setFocusProxy(d->m_lineEdit);
@@ -299,18 +293,23 @@ void PathChooser::addButton(const QString &text, QObject *context, const std::fu
     insertButton(d->m_buttons.count(), text, context, callback);
 }
 
+void PathChooser::insertButton(int index, QAbstractButton *button)
+{
+    d->m_hLayout->insertWidget(index + 1 /*line edit*/, button);
+    d->m_buttons.insert(index, button);
+}
+
 void PathChooser::insertButton(int index, const QString &text, QObject *context, const std::function<void ()> &callback)
 {
     auto button = new QPushButton;
     button->setText(text);
     connect(button, &QAbstractButton::clicked, context, callback);
-    d->m_hLayout->insertWidget(index + 1/*line edit*/, button);
-    d->m_buttons.insert(index, button);
+    insertButton(index, button);
 }
 
 QString PathChooser::browseButtonLabel()
 {
-    return HostOsInfo::isMacHost() ? tr("Choose...") : tr("Browse...");
+    return HostOsInfo::isMacHost() ? Tr::tr("Choose...") : Tr::tr("Browse...");
 }
 
 QAbstractButton *PathChooser::buttonAtIndex(int index) const
@@ -331,21 +330,16 @@ FilePath PathChooser::baseDirectory() const
     return d->m_baseDirectory;
 }
 
-void PathChooser::setEnvironmentChange(const EnvironmentChange &env)
+void PathChooser::setEnvironment(const Environment &env)
 {
     QString oldExpand = filePath().toString();
-    d->m_environmentChange = env;
+    d->m_environment = env;
     if (filePath().toString() != oldExpand) {
         triggerChanged();
-        emit rawPathChanged(rawPath());
+        emit rawPathChanged();
     }
 }
 
-QString PathChooser::rawPath() const
-{
-    return rawFilePath().toString();
-}
-
 FilePath PathChooser::rawFilePath() const
 {
     return FilePath::fromUserInput(d->m_lineEdit->text());
@@ -361,27 +355,17 @@ FilePath PathChooser::absoluteFilePath() const
     return d->m_baseDirectory.resolvePath(filePath());
 }
 
-// FIXME: try to remove again
-QString PathChooser::expandedDirectory(const QString &input, const Environment &env,
-                                       const QString &baseDir)
-{
-    if (input.isEmpty())
-        return input;
-    const QString path = QDir::cleanPath(env.expandVariables(input));
-    if (path.isEmpty())
-        return path;
-    if (!baseDir.isEmpty() && QFileInfo(path).isRelative())
-        return QFileInfo(baseDir + '/' + path).absoluteFilePath();
-    return path;
-}
-
 void PathChooser::setPath(const QString &path)
 {
+    QTC_ASSERT(!d->m_callGuard.isLocked(), return);
+    GuardLocker locker(d->m_callGuard);
     d->m_lineEdit->setTextKeepingActiveCursor(QDir::toNativeSeparators(path));
 }
 
 void PathChooser::setFilePath(const FilePath &fn)
 {
+    QTC_ASSERT(!d->m_callGuard.isLocked(), return);
+    GuardLocker locker(d->m_callGuard);
     d->m_lineEdit->setTextKeepingActiveCursor(fn.toUserOutput());
 }
 
@@ -393,12 +377,13 @@ bool PathChooser::isReadOnly() const
 void PathChooser::setReadOnly(bool b)
 {
     d->m_lineEdit->setReadOnly(b);
+    d->m_lineEdit->setFrame(!b);
     const auto buttons = d->m_buttons;
     for (QAbstractButton *button : buttons)
-        button->setEnabled(!b);
+        button->setVisible(!b);
 }
 
-void PathChooser::slotBrowse()
+void PathChooser::slotBrowse(bool remote)
 {
     emit beforeBrowsing();
 
@@ -414,46 +399,63 @@ void PathChooser::slotBrowse()
             predefined.clear();
     }
 
+    remote = remote || filePath().needsDevice();
+
     // Prompt for a file/dir
     FilePath newPath;
     switch (d->m_acceptingKind) {
     case PathChooser::Directory:
     case PathChooser::ExistingDirectory:
         newPath = FileUtils::getExistingDirectory(this,
-                        makeDialogTitle(tr("Choose Directory")), predefined);
+                                                  makeDialogTitle(Tr::tr("Choose Directory")),
+                                                  predefined,
+                                                  {},
+                                                  d->m_allowPathFromDevice,
+                                                  remote);
         break;
     case PathChooser::ExistingCommand:
     case PathChooser::Command:
         newPath = FileUtils::getOpenFilePath(this,
-                        makeDialogTitle(tr("Choose Executable")), predefined, d->m_dialogFilter);
+                                             makeDialogTitle(Tr::tr("Choose Executable")),
+                                             predefined,
+                                             d->m_dialogFilter,
+                                             nullptr,
+                                             {},
+                                             d->m_allowPathFromDevice,
+                                             remote);
         newPath = appBundleExpandedPath(newPath);
         break;
     case PathChooser::File: // fall through
         newPath = FileUtils::getOpenFilePath(this,
-                        makeDialogTitle(tr("Choose File")), predefined, d->m_dialogFilter);
+                                             makeDialogTitle(Tr::tr("Choose File")),
+                                             predefined,
+                                             d->m_dialogFilter,
+                                             nullptr,
+                                             {},
+                                             d->m_allowPathFromDevice,
+                                             remote);
         newPath = appBundleExpandedPath(newPath);
         break;
     case PathChooser::SaveFile:
         newPath = FileUtils::getSaveFilePath(this,
-                        makeDialogTitle(tr("Choose File")), predefined, d->m_dialogFilter);
+                                             makeDialogTitle(Tr::tr("Choose File")),
+                                             predefined,
+                                             d->m_dialogFilter,
+                                             nullptr,
+                                             {},
+                                             remote);
         break;
     case PathChooser::Any: {
-        QFileDialog dialog(this);
-        dialog.setFileMode(QFileDialog::AnyFile);
-        dialog.setWindowTitle(makeDialogTitle(tr("Choose File")));
-        if (predefined.exists())
-            dialog.setDirectory(predefined.absolutePath().toDir());
-        // FIXME: fix QFileDialog so that it filters properly: lib*.a
-        dialog.setNameFilter(d->m_dialogFilter);
-        if (dialog.exec() == QDialog::Accepted) {
-            // probably loop here until the *.framework dir match
-            QStringList paths = dialog.selectedFiles();
-            if (!paths.isEmpty())
-                newPath = FilePath::fromString(paths.at(0));
-        }
+        newPath = FileUtils::getOpenFilePath(this,
+                                             makeDialogTitle(Tr::tr("Choose File")),
+                                             predefined,
+                                             d->m_dialogFilter,
+                                             nullptr,
+                                             {},
+                                             d->m_allowPathFromDevice,
+                                             remote);
         break;
-        }
-
+    }
     default:
         break;
     }
@@ -522,100 +524,83 @@ void PathChooser::setDefaultValue(const QString &defaultValue)
     d->m_lineEdit->validate();
 }
 
-FancyLineEdit::ValidationFunction PathChooser::defaultValidationFunction() const
+void PathChooser::setPlaceholderText(const QString &placeholderText)
 {
-    return std::bind(&PathChooser::validatePath, this, std::placeholders::_1, std::placeholders::_2);
+    d->m_lineEdit->setPlaceholderText(placeholderText);
 }
 
-bool PathChooser::validatePath(FancyLineEdit *edit, QString *errorMessage) const
+void PathChooser::setToolTip(const QString &toolTip)
 {
-    QString input = edit->text();
+    d->m_lineEdit->setToolTip(toolTip);
+}
 
-    if (input.isEmpty()) {
-        if (!d->m_defaultValue.isEmpty()) {
-            input = d->m_defaultValue;
-        } else {
-            if (errorMessage)
-                *errorMessage = tr("The path must not be empty.");
-            return false;
-        }
-    }
-
-    const FilePath filePath = d->expandedPath(FilePath::fromUserInput(input));
+static FancyLineEdit::AsyncValidationResult validatePath(FilePath filePath,
+                                                         const QString &defaultValue,
+                                                         PathChooser::Kind kind)
+{
     if (filePath.isEmpty()) {
-        if (errorMessage)
-            *errorMessage = tr("The path \"%1\" expanded to an empty string.").arg(input);
-        return false;
+        if (!defaultValue.isEmpty()) {
+            filePath = FilePath::fromUserInput(defaultValue);
+        } else {
+            return make_unexpected(Tr::tr("The path must not be empty."));
+        }
     }
 
     // Check if existing
-    switch (d->m_acceptingKind) {
+    switch (kind) {
     case PathChooser::ExistingDirectory:
         if (!filePath.exists()) {
-            if (errorMessage)
-                *errorMessage = tr("The path \"%1\" does not exist.").arg(filePath.toUserOutput());
-            return false;
+            return make_unexpected(
+                Tr::tr("The path \"%1\" does not exist.").arg(filePath.toUserOutput()));
         }
         if (!filePath.isDir()) {
-            if (errorMessage)
-                *errorMessage = tr("The path \"%1\" is not a directory.").arg(filePath.toUserOutput());
-            return false;
+            return make_unexpected(
+                Tr::tr("The path \"%1\" is not a directory.").arg(filePath.toUserOutput()));
         }
         break;
     case PathChooser::File:
         if (!filePath.exists()) {
-            if (errorMessage)
-                *errorMessage = tr("The path \"%1\" does not exist.").arg(filePath.toUserOutput());
-            return false;
+            return make_unexpected(
+                Tr::tr("The path \"%1\" does not exist.").arg(filePath.toUserOutput()));
         }
         if (!filePath.isFile()) {
-            if (errorMessage)
-                *errorMessage = tr("The path \"%1\" is not a file.").arg(filePath.toUserOutput());
-            return false;
+            return make_unexpected(
+                Tr::tr("The path \"%1\" is not a file.").arg(filePath.toUserOutput()));
         }
         break;
     case PathChooser::SaveFile:
         if (!filePath.parentDir().exists()) {
-            if (errorMessage)
-                *errorMessage = tr("The directory \"%1\" does not exist.").arg(filePath.toUserOutput());
-            return false;
+            return make_unexpected(
+                Tr::tr("The directory \"%1\" does not exist.").arg(filePath.toUserOutput()));
         }
         if (filePath.exists() && filePath.isDir()) {
-            if (errorMessage)
-                *errorMessage = tr("The path \"%1\" is not a file.").arg(filePath.toUserOutput());
-            return false;
+            return make_unexpected(
+                Tr::tr("The path \"%1\" is not a file.").arg(filePath.toUserOutput()));
         }
         break;
     case PathChooser::ExistingCommand:
         if (!filePath.exists()) {
-            if (errorMessage)
-                *errorMessage = tr("The path \"%1\" does not exist.").arg(filePath.toUserOutput());
-            return false;
+            return make_unexpected(
+                Tr::tr("The path \"%1\" does not exist.").arg(filePath.toUserOutput()));
         }
         if (!filePath.isExecutableFile()) {
-            if (errorMessage)
-                *errorMessage = tr("The path \"%1\" is not an executable file.").arg(filePath.toUserOutput());
-            return false;
+            return make_unexpected(
+                Tr::tr("The path \"%1\" is not an executable file.").arg(filePath.toUserOutput()));
         }
         break;
     case PathChooser::Directory:
         if (filePath.exists() && !filePath.isDir()) {
-            if (errorMessage)
-                *errorMessage = tr("The path \"%1\" is not a directory.").arg(filePath.toUserOutput());
-            return false;
+            return make_unexpected(
+                Tr::tr("The path \"%1\" is not a directory.").arg(filePath.toUserOutput()));
         }
-        if (HostOsInfo::isWindowsHost() && !filePath.startsWithDriveLetter()
-                && !filePath.startsWith("\\\\") && !filePath.startsWith("//")) {
-            if (errorMessage)
-                *errorMessage = tr("Invalid path \"%1\".").arg(filePath.toUserOutput());
-            return false;
+        if (filePath.osType() == OsTypeWindows && !filePath.startsWithDriveLetter()
+            && !filePath.startsWith("\\\\") && !filePath.startsWith("//")) {
+            return make_unexpected(Tr::tr("Invalid path \"%1\".").arg(filePath.toUserOutput()));
         }
         break;
     case PathChooser::Command:
         if (filePath.exists() && !filePath.isExecutableFile()) {
-            if (errorMessage)
-                *errorMessage = tr("Cannot execute \"%1\".").arg(filePath.toUserOutput());
-            return false;
+            return make_unexpected(Tr::tr("Cannot execute \"%1\".").arg(filePath.toUserOutput()));
         }
         break;
 
@@ -623,9 +608,32 @@ bool PathChooser::validatePath(FancyLineEdit *edit, QString *errorMessage) const
         ;
     }
 
-    if (errorMessage)
-        *errorMessage = tr("Full path: \"%1\"").arg(filePath.toUserOutput());
-    return true;
+    return filePath.toUserOutput();
+}
+
+FancyLineEdit::AsyncValidationFunction PathChooser::defaultValidationFunction() const
+{
+    return [this](const QString &text) -> FancyLineEdit::AsyncValidationFuture {
+        if (text.isEmpty()) {
+            return QtFuture::makeReadyFuture((Utils::expected_str<QString>(
+                make_unexpected(Tr::tr("The path must not be empty.")))));
+        }
+
+        const FilePath expanded = d->expandedPath(FilePath::fromUserInput(text));
+
+        if (expanded.isEmpty()) {
+            return QtFuture::makeReadyFuture((Utils::expected_str<QString>(
+                make_unexpected(Tr::tr("The path \"%1\" expanded to an empty string.")
+                                    .arg(expanded.toUserOutput())))));
+        }
+
+        return Utils::asyncRun(
+            [expanded,
+             defVal = d->m_defaultValue,
+             kind = d->m_acceptingKind]() -> FancyLineEdit::AsyncValidationResult {
+                return validatePath(expanded, defVal, kind);
+            });
+    };
 }
 
 void PathChooser::setValidationFunction(const FancyLineEdit::ValidationFunction &fn)
@@ -635,7 +643,7 @@ void PathChooser::setValidationFunction(const FancyLineEdit::ValidationFunction
 
 QString PathChooser::label()
 {
-    return tr("Path:");
+    return Tr::tr("Path:");
 }
 
 FilePath PathChooser::homePath()
@@ -727,7 +735,7 @@ void PathChooser::installLineEditVersionToolTip(QLineEdit *le, const QStringList
     ef->setArguments(arguments);
 }
 
-void PathChooser::setHistoryCompleter(const QString &historyKey, bool restoreLastItemFromHistory)
+void PathChooser::setHistoryCompleter(const Key &historyKey, bool restoreLastItemFromHistory)
 {
     d->m_lineEdit->setHistoryCompleter(historyKey, restoreLastItemFromHistory);
 }
@@ -758,4 +766,19 @@ void PathChooser::setCommandVersionArguments(const QStringList &arguments)
     }
 }
 
+void PathChooser::setAllowPathFromDevice(bool allow)
+{
+    d->m_allowPathFromDevice = allow;
+
+    if (allow && FileUtils::hasNativeFileDialog())
+        d->m_browseButton->setOptionalMenu(d->m_contextMenu);
+    else
+        d->m_browseButton->setOptionalMenu(nullptr);
+}
+
+bool PathChooser::allowPathFromDevice() const
+{
+    return d->m_allowPathFromDevice;
+}
+
 } // namespace Utils
