@@ -1,30 +1,10 @@
-/****************************************************************************
-**
-** Copyright (C) 2016 The Qt Company Ltd.
-** Contact: https://www.qt.io/licensing/
-**
-** This file is part of Qt Creator.
-**
-** Commercial License Usage
-** Licensees holding valid commercial Qt licenses may use this file in
-** accordance with the commercial license agreement provided with the
-** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and The Qt Company. For licensing terms
-** and conditions see https://www.qt.io/terms-conditions. For further
-** information use the contact form at https://www.qt.io/contact-us.
-**
-** GNU General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU
-** General Public License version 3 as published by the Free Software
-** Foundation with exceptions as appearing in the file LICENSE.GPL3-EXCEPT
-** included in the packaging of this file. Please review the following
-** information to ensure the GNU General Public License requirements will
-** be met: https://www.gnu.org/licenses/gpl-3.0.html.
-**
-****************************************************************************/
+// Copyright (C) 2016 The Qt Company Ltd.
+// SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only WITH Qt-GPL-exception-1.0
 
 #include "treemodel.h"
+
 #include "qtcassert.h"
+#include "stringutils.h"
 
 #include <QStack>
 #include <QSize>
@@ -93,6 +73,7 @@ private:
 };
 
 /*!
+    \internal
     Connect to all of the models signals.  Whenever anything happens
     recheck everything.
 */
@@ -156,6 +137,7 @@ void ModelTest::runAllTests()
 }
 
 /*!
+    \internal
     nonDestructiveBasicTest tries to call a number of the basic functions (not all)
     to make sure the model doesn't outright segfault, testing the functions that makes sense.
 */
@@ -194,6 +176,7 @@ void ModelTest::nonDestructiveBasicTest()
 }
 
 /*!
+    \internal
     Tests model's implementation of QAbstractItemModel::rowCount() and hasChildren()
 
     Models that are dynamically populated are not as fully tested here.
@@ -221,6 +204,7 @@ void ModelTest::rowCount()
 }
 
 /*!
+    \internal
     Tests model's implementation of QAbstractItemModel::columnCount() and hasChildren()
  */
 void ModelTest::columnCount()
@@ -239,6 +223,7 @@ void ModelTest::columnCount()
 }
 
 /*!
+    \internal
     Tests model's implementation of QAbstractItemModel::hasIndex()
  */
 void ModelTest::hasIndex()
@@ -263,6 +248,7 @@ void ModelTest::hasIndex()
 }
 
 /*!
+    \internal
     Tests model's implementation of QAbstractItemModel::index()
  */
 void ModelTest::index()
@@ -295,6 +281,7 @@ void ModelTest::index()
 }
 
 /*!
+    \internal
     Tests model's implementation of QAbstractItemModel::parent()
  */
 void ModelTest::parent()
@@ -343,6 +330,7 @@ void ModelTest::parent()
 }
 
 /*!
+    \internal
     Called from the parent() test.
 
     A model that returns an index of parent X should also return X when asking
@@ -451,6 +439,7 @@ void ModelTest::checkChildren(const QModelIndex &parent, int currentDepth)
 }
 
 /*!
+    \internal
     Tests model's implementation of QAbstractItemModel::data()
  */
 void ModelTest::data()
@@ -515,13 +504,14 @@ void ModelTest::data()
 }
 
 /*!
+    \internal
     Store what is about to be inserted to make sure it actually happens
 
     \sa rowsInserted()
  */
 void ModelTest::rowsAboutToBeInserted(const QModelIndex &parent, int start, int end)
 {
-    Q_UNUSED(end);
+    Q_UNUSED(end)
     Changing c;
     c.parent = parent;
     c.oldSize = model->rowCount(parent);
@@ -531,6 +521,7 @@ void ModelTest::rowsAboutToBeInserted(const QModelIndex &parent, int start, int
 }
 
 /*!
+    \internal
     Confirm that what was said was going to happen actually did
 
     \sa rowsAboutToBeInserted()
@@ -568,6 +559,7 @@ void ModelTest::layoutChanged()
 }
 
 /*!
+    \internal
     Store what is about to be inserted to make sure it actually happens
 
     \sa rowsRemoved()
@@ -583,6 +575,7 @@ void ModelTest::rowsAboutToBeRemoved(const QModelIndex &parent, int start, int e
 }
 
 /*!
+    \internal
     Confirm that what was said was going to happen actually did
 
     \sa rowsAboutToBeRemoved()
@@ -628,22 +621,22 @@ int TreeItem::indexOf(const TreeItem *item) const
 
 QVariant TreeItem::data(int column, int role) const
 {
-    Q_UNUSED(column);
-    Q_UNUSED(role);
+    Q_UNUSED(column)
+    Q_UNUSED(role)
     return QVariant();
 }
 
 bool TreeItem::setData(int column, const QVariant &data, int role)
 {
-    Q_UNUSED(column);
-    Q_UNUSED(data);
-    Q_UNUSED(role);
+    Q_UNUSED(column)
+    Q_UNUSED(data)
+    Q_UNUSED(role)
     return false;
 }
 
 Qt::ItemFlags TreeItem::flags(int column) const
 {
-    Q_UNUSED(column);
+    Q_UNUSED(column)
     return Qt::ItemIsEnabled | Qt::ItemIsSelectable;
 }
 
@@ -678,11 +671,11 @@ void TreeItem::insertChild(int pos, TreeItem *item)
         m_model->beginInsertRows(idx, pos, pos);
         item->m_parent = this;
         item->propagateModel(m_model);
-        m_children.insert(m_children.begin() + pos, item);
+        m_children.insert(m_children.cbegin() + pos, item);
         m_model->endInsertRows();
     } else {
         item->m_parent = this;
-        m_children.insert(m_children.begin() + pos, item);
+        m_children.insert(m_children.cbegin() + pos, item);
     }
 }
 
@@ -725,7 +718,7 @@ void TreeItem::sortChildren(const std::function<bool(const TreeItem *, const Tre
 {
     if (m_model) {
         if (const int n = childCount()) {
-            QVector<TreeItem *> tmp = m_children;
+            QList<TreeItem *> tmp = m_children;
             std::sort(tmp.begin(), tmp.end(), cmp);
             if (tmp == m_children) {
                 // Nothing changed.
@@ -748,7 +741,8 @@ void TreeItem::update()
 {
     if (m_model) {
         QModelIndex idx = index();
-        m_model->dataChanged(idx.sibling(idx.row(), 0), idx.sibling(idx.row(), m_model->m_columnCount - 1));
+        emit m_model->dataChanged(idx.sibling(idx.row(), 0),
+                                  idx.sibling(idx.row(), m_model->m_columnCount - 1));
     }
 }
 
@@ -756,7 +750,7 @@ void TreeItem::updateAll()
 {
     if (m_model) {
         QModelIndex idx = index();
-        m_model->dataChanged(idx, idx.sibling(idx.row(), m_model->m_columnCount - 1));
+        emit m_model->dataChanged(idx, idx.sibling(idx.row(), m_model->m_columnCount - 1));
         for (TreeItem *item : *this)
             item->updateAll();
     }
@@ -766,7 +760,7 @@ void TreeItem::updateColumn(int column)
 {
     if (m_model) {
         QModelIndex idx = index();
-        m_model->dataChanged(idx.sibling(idx.row(), column), idx.sibling(idx.row(), column));
+        emit m_model->dataChanged(idx.sibling(idx.row(), column), idx.sibling(idx.row(), column));
     }
 }
 
@@ -893,13 +887,13 @@ void TreeItem::removeItemAt(int pos)
 void TreeItem::expand()
 {
     QTC_ASSERT(m_model, return);
-    m_model->requestExpansion(index());
+    emit m_model->requestExpansion(index());
 }
 
 void TreeItem::collapse()
 {
     QTC_ASSERT(m_model, return);
-    m_model->requestCollapse(index());
+    emit m_model->requestCollapse(index());
 }
 
 void TreeItem::propagateModel(BaseTreeModel *m)
@@ -915,6 +909,7 @@ void TreeItem::propagateModel(BaseTreeModel *m)
 
 /*!
     \class Utils::TreeModel
+    \inmodule QtCreator
 
     \brief The TreeModel class is a convienience base class for models
     to use in a QTreeView.
@@ -1036,7 +1031,7 @@ bool BaseTreeModel::hasChildren(const QModelIndex &idx) const
 Qt::ItemFlags BaseTreeModel::flags(const QModelIndex &idx) const
 {
     if (!idx.isValid())
-        return nullptr;
+        return {};
     TreeItem *item = itemForIndex(idx);
     return item ? item->flags(idx.column())
                 : (Qt::ItemIsEnabled|Qt::ItemIsSelectable);
@@ -1185,17 +1180,32 @@ StaticTreeItem::StaticTreeItem(const QString &display)
 {
 }
 
+StaticTreeItem::StaticTreeItem(const QStringList &displays, const QStringList &toolTips)
+    : m_displays(displays)
+    , m_toolTips(toolTips)
+{}
+
 QVariant StaticTreeItem::data(int column, int role) const
 {
     if (role == Qt::DisplayRole && column >= 0 && column < m_displays.size())
         return m_displays.at(column);
+    if (role == Qt::ToolTipRole && column >= 0 && column < m_toolTips.size())
+        return m_toolTips.at(column);
     return QVariant();
 }
 
 Qt::ItemFlags StaticTreeItem::flags(int column) const
 {
-    Q_UNUSED(column);
+    Q_UNUSED(column)
     return Qt::ItemIsEnabled;
 }
 
+bool SortModel::lessThan(const QModelIndex &source_left, const QModelIndex &source_right) const
+{
+    if (m_lessThan)
+        return m_lessThan(source_left, source_right);
+    return caseFriendlyCompare(sourceModel()->data(source_left).toString(),
+                               sourceModel()->data(source_right).toString()) < 0;
+}
+
 } // namespace Utils
