@@ -1,35 +1,11 @@
-/****************************************************************************
-**
-** Copyright (C) 2016 The Qt Company Ltd.
-** Contact: https://www.qt.io/licensing/
-**
-** This file is part of Qt Creator.
-**
-** Commercial License Usage
-** Licensees holding valid commercial Qt licenses may use this file in
-** accordance with the commercial license agreement provided with the
-** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and The Qt Company. For licensing terms
-** and conditions see https://www.qt.io/terms-conditions. For further
-** information use the contact form at https://www.qt.io/contact-us.
-**
-** GNU General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU
-** General Public License version 3 as published by the Free Software
-** Foundation with exceptions as appearing in the file LICENSE.GPL3-EXCEPT
-** included in the packaging of this file. Please review the following
-** information to ensure the GNU General Public License requirements will
-** be met: https://www.gnu.org/licenses/gpl-3.0.html.
-**
-****************************************************************************/
+// Copyright (C) 2016 The Qt Company Ltd.
+// SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only WITH Qt-GPL-exception-1.0
 
 #include "procinterrupt.h"
 #include "debuggerconstants.h"
 
-#include <app/app_version.h>
-
-#include <QCoreApplication>
 #include <QDir>
+#include <QGuiApplication>
 #include <QProcess> // makes kill visible on Windows.
 
 using namespace Debugger::Internal;
@@ -40,10 +16,6 @@ static inline QString msgCannotInterrupt(qint64 pid, const QString &why)
 }
 
 #if defined(Q_OS_WIN)
-
-#undef _WIN32_WINNT
-#define _WIN32_WINNT 0x0501 /* WinXP, needed for DebugBreakProcess() */
-
 #include <utils/winutils.h>
 #include <windows.h>
 
@@ -148,11 +120,13 @@ GDB 32bit | Api             | Api             | NA              | Win32
             const QString executable = breakApi == UseWin32Interrupt
                     ? QCoreApplication::applicationDirPath() + "/win32interrupt.exe"
                     : QCoreApplication::applicationDirPath() + "/win64interrupt.exe";
-            if (!QFile::exists(executable)) {
-                *errorMessage = QString::fromLatin1("%1 does not exist. If you have built %2 "
-                                                    "on your own, checkout "
-                                                    "https://code.qt.io/cgit/qt-creator/binary-artifacts.git/.").
-                        arg(QDir::toNativeSeparators(executable), Core::Constants::IDE_DISPLAY_NAME);
+            if (!QFileInfo::exists(executable)) {
+                *errorMessage = QString::fromLatin1(
+                                    "%1 does not exist. If you have built %2 "
+                                    "on your own, checkout "
+                                    "https://code.qt.io/cgit/qt-creator/binary-artifacts.git/.")
+                                    .arg(QDir::toNativeSeparators(executable),
+                                         QGuiApplication::applicationDisplayName());
                 break;
             }
             switch (QProcess::execute(executable, QStringList(QString::number(pID)))) {
