@@ -1,41 +1,20 @@
-/****************************************************************************
-**
-** Copyright (C) 2018 The Qt Company Ltd.
-** Contact: https://www.qt.io/licensing/
-**
-** This file is part of Qt Creator.
-**
-** Commercial License Usage
-** Licensees holding valid commercial Qt licenses may use this file in
-** accordance with the commercial license agreement provided with the
-** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and The Qt Company. For licensing terms
-** and conditions see https://www.qt.io/terms-conditions. For further
-** information use the contact form at https://www.qt.io/contact-us.
-**
-** GNU General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU
-** General Public License version 3 as published by the Free Software
-** Foundation with exceptions as appearing in the file LICENSE.GPL3-EXCEPT
-** included in the packaging of this file. Please review the following
-** information to ensure the GNU General Public License requirements will
-** be met: https://www.gnu.org/licenses/gpl-3.0.html.
-**
-****************************************************************************/
+// Copyright (C) 2018 The Qt Company Ltd.
+// SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only WITH Qt-GPL-exception-1.0
 
 #include "languageclientsettings.h"
 
 #include "client.h"
-#include "languageclientmanager.h"
 #include "languageclient_global.h"
 #include "languageclientinterface.h"
+#include "languageclientmanager.h"
+#include "languageclienttr.h"
 
 #include <coreplugin/editormanager/documentmodel.h>
 #include <coreplugin/icore.h>
 #include <coreplugin/idocument.h>
 
 #include <projectexplorer/project.h>
-#include <projectexplorer/session.h>
+#include <projectexplorer/projectmanager.h>
 
 #include <texteditor/plaintexteditorfactory.h>
 #include <texteditor/textmark.h>
@@ -57,6 +36,7 @@
 #include <QDialogButtonBox>
 #include <QDir>
 #include <QFileInfo>
+#include <QGroupBox>
 #include <QHeaderView>
 #include <QJsonDocument>
 #include <QLabel>
@@ -64,7 +44,6 @@
 #include <QMenu>
 #include <QMimeData>
 #include <QPushButton>
-#include <QSettings>
 #include <QSortFilterProxyModel>
 #include <QStringListModel>
 #include <QToolButton>
@@ -87,6 +66,8 @@ constexpr char typedClientsKey[] = "typedClients";
 constexpr char outlineSortedKey[] = "outlineSorted";
 constexpr char mimeType[] = "application/language.client.setting";
 
+using namespace Utils;
+
 namespace LanguageClient {
 
 class LanguageClientSettingsModel : public QAbstractListModel
@@ -125,17 +106,38 @@ private:
     QList<BaseSettings *> m_removed;
 };
 
-class LanguageClientSettingsPageWidget : public QWidget
+class LanguageClientSettingsPageWidget : public Core::IOptionsPageWidget
 {
 public:
-    LanguageClientSettingsPageWidget(LanguageClientSettingsModel &settings);
+    LanguageClientSettingsPageWidget(LanguageClientSettingsModel &settings,
+                                     QSet<QString> &changedSettings);
+
     void currentChanged(const QModelIndex &index);
     int currentRow() const;
     void resetCurrentSettings(int row);
     void applyCurrentSettings();
 
+    void apply() final
+    {
+        applyCurrentSettings();
+        LanguageClientManager::applySettings();
+
+        for (BaseSettings *setting : m_settings.removed()) {
+            for (Client *client : LanguageClientManager::clientsForSetting(setting))
+                LanguageClientManager::shutdownClient(client);
+        }
+
+        int row = currentRow();
+        m_settings.reset(LanguageClientManager::currentSettings());
+        resetCurrentSettings(row);
+    }
+    void finish()
+    {
+        m_settings.reset(LanguageClientManager::currentSettings());
+        m_changedSettings.clear();
+    }
+
 private:
-    LanguageClientSettingsModel &m_settings;
     QTreeView *m_view = nullptr;
     struct CurrentSettings {
         BaseSettings *setting = nullptr;
@@ -144,31 +146,9 @@ private:
 
     void addItem(const Utils::Id &clientTypeId);
     void deleteItem();
-};
 
-class LanguageClientSettingsPage : public Core::IOptionsPage
-{
-    Q_DECLARE_TR_FUNCTIONS(LanguageClientSettingsPage)
-public:
-    LanguageClientSettingsPage();
-    ~LanguageClientSettingsPage() override;
-
-    void init();
-
-    // IOptionsPage interface
-    QWidget *widget() override;
-    void apply() override;
-    void finish() override;
-
-    QList<BaseSettings *> settings() const;
-    QList<BaseSettings *> changedSettings() const;
-    void addSettings(BaseSettings *settings);
-    void enableSettings(const QString &id, bool enable = true);
-
-private:
-    LanguageClientSettingsModel m_model;
-    QSet<QString> m_changedSettings;
-    QPointer<LanguageClientSettingsPageWidget> m_widget;
+    LanguageClientSettingsModel &m_settings;
+    QSet<QString> &m_changedSettings;
 };
 
 QMap<Utils::Id, ClientType> &clientTypes()
@@ -177,9 +157,11 @@ QMap<Utils::Id, ClientType> &clientTypes()
     return types;
 }
 
-LanguageClientSettingsPageWidget::LanguageClientSettingsPageWidget(LanguageClientSettingsModel &settings)
-    : m_settings(settings)
-    , m_view(new QTreeView())
+LanguageClientSettingsPageWidget::LanguageClientSettingsPageWidget(LanguageClientSettingsModel &settings,
+                                                                   QSet<QString> &changedSettings)
+    : m_view(new QTreeView())
+    , m_settings(settings)
+    , m_changedSettings(changedSettings)
 {
     auto mainLayout = new QVBoxLayout();
     auto layout = new QHBoxLayout();
@@ -194,7 +176,7 @@ LanguageClientSettingsPageWidget::LanguageClientSettingsPageWidget(LanguageClien
     connect(m_view->selectionModel(), &QItemSelectionModel::currentChanged,
             this, &LanguageClientSettingsPageWidget::currentChanged);
     auto buttonLayout = new QVBoxLayout();
-    auto addButton = new QPushButton(LanguageClientSettingsPage::tr("&Add"));
+    auto addButton = new QPushButton(Tr::tr("&Add"));
     auto addMenu = new QMenu;
     addMenu->clear();
     for (const ClientType &type : clientTypes()) {
@@ -203,7 +185,7 @@ LanguageClientSettingsPageWidget::LanguageClientSettingsPageWidget(LanguageClien
         addMenu->addAction(action);
     }
     addButton->setMenu(addMenu);
-    auto deleteButton = new QPushButton(LanguageClientSettingsPage::tr("&Delete"));
+    auto deleteButton = new QPushButton(Tr::tr("&Delete"));
     connect(deleteButton, &QPushButton::pressed, this, &LanguageClientSettingsPageWidget::deleteItem);
     mainLayout->addLayout(layout);
     setLayout(mainLayout);
@@ -286,64 +268,40 @@ void LanguageClientSettingsPageWidget::deleteItem()
     m_settings.removeRows(index.row());
 }
 
+class LanguageClientSettingsPage : public Core::IOptionsPage
+{
+public:
+    LanguageClientSettingsPage();
+
+    void init();
+
+    QList<BaseSettings *> settings() const;
+    QList<BaseSettings *> changedSettings() const;
+    void addSettings(BaseSettings *settings);
+    void enableSettings(const QString &id, bool enable = true);
+
+private:
+    LanguageClientSettingsModel m_model;
+    QSet<QString> m_changedSettings;
+};
+
 LanguageClientSettingsPage::LanguageClientSettingsPage()
 {
     setId(Constants::LANGUAGECLIENT_SETTINGS_PAGE);
-    setDisplayName(tr("General"));
+    setDisplayName(Tr::tr("General"));
     setCategory(Constants::LANGUAGECLIENT_SETTINGS_CATEGORY);
-    setDisplayCategory(QCoreApplication::translate("LanguageClient",
-                                                   Constants::LANGUAGECLIENT_SETTINGS_TR));
+    setDisplayCategory(Tr::tr(Constants::LANGUAGECLIENT_SETTINGS_TR));
     setCategoryIconPath(":/languageclient/images/settingscategory_languageclient.png");
-    connect(&m_model, &LanguageClientSettingsModel::dataChanged, [this](const QModelIndex &index) {
+    setWidgetCreator([this] { return new LanguageClientSettingsPageWidget(m_model, m_changedSettings); });
+    QObject::connect(&m_model, &LanguageClientSettingsModel::dataChanged, [this](const QModelIndex &index) {
         if (BaseSettings *setting = m_model.settingForIndex(index))
             m_changedSettings << setting->m_id;
     });
 }
 
-LanguageClientSettingsPage::~LanguageClientSettingsPage()
-{
-    if (m_widget)
-        delete m_widget;
-}
-
 void LanguageClientSettingsPage::init()
 {
     m_model.reset(LanguageClientSettings::fromSettings(Core::ICore::settings()));
-    apply();
-    finish();
-}
-
-QWidget *LanguageClientSettingsPage::widget()
-{
-    if (!m_widget)
-        m_widget = new LanguageClientSettingsPageWidget(m_model);
-    return m_widget;
-}
-
-void LanguageClientSettingsPage::apply()
-{
-    if (m_widget)
-        m_widget->applyCurrentSettings();
-    LanguageClientManager::applySettings();
-
-    for (BaseSettings *setting : m_model.removed()) {
-        for (Client *client : LanguageClientManager::clientsForSetting(setting))
-            LanguageClientManager::shutdownClient(client);
-    }
-
-    if (m_widget) {
-        int row = m_widget->currentRow();
-        m_model.reset(LanguageClientManager::currentSettings());
-        m_widget->resetCurrentSettings(row);
-    } else {
-        m_model.reset(LanguageClientManager::currentSettings());
-    }
-}
-
-void LanguageClientSettingsPage::finish()
-{
-    m_model.reset(LanguageClientManager::currentSettings());
-    m_changedSettings.clear();
 }
 
 QList<BaseSettings *> LanguageClientSettingsPage::settings() const
@@ -605,9 +563,9 @@ Client *BaseSettings::createClient(BaseClientInterface *interface) const
     return new Client(interface);
 }
 
-QVariantMap BaseSettings::toMap() const
+Store BaseSettings::toMap() const
 {
-    QVariantMap map;
+    Store map;
     map.insert(typeIdKey, m_settingsTypeId.toSetting());
     map.insert(nameKey, m_name);
     map.insert(idKey, m_id);
@@ -620,7 +578,7 @@ QVariantMap BaseSettings::toMap() const
     return map;
 }
 
-void BaseSettings::fromMap(const QVariantMap &map)
+void BaseSettings::fromMap(const Store &map)
 {
     m_name = map[nameKey].toString();
     m_id = map.value(idKey, QUuid::createUuid().toString()).toString();
@@ -643,18 +601,19 @@ static LanguageClientSettingsPage &settingsPage()
 void LanguageClientSettings::init()
 {
     settingsPage().init();
+    LanguageClientManager::applySettings();
 }
 
-QList<BaseSettings *> LanguageClientSettings::fromSettings(QSettings *settingsIn)
+QList<BaseSettings *> LanguageClientSettings::fromSettings(QtcSettings *settingsIn)
 {
     settingsIn->beginGroup(settingsGroupKey);
     QList<BaseSettings *> result;
 
-    for (auto varList :
+    for (const QVariantList &varList :
          {settingsIn->value(clientsKey).toList(), settingsIn->value(typedClientsKey).toList()}) {
         for (const QVariant &var : varList) {
-            const QMap<QString, QVariant> &map = var.toMap();
-            Utils::Id typeId = Utils::Id::fromSetting(map.value(typeIdKey));
+            const Store map = storeFromVariant(var);
+            Id typeId = Id::fromSetting(map.value(typeIdKey));
             if (!typeId.isValid())
                 typeId = Constants::LANGUAGECLIENT_STDIO_SETTINGS_ID;
             if (BaseSettings *settings = generateSettings(typeId)) {
@@ -694,13 +653,13 @@ void LanguageClientSettings::enableSettings(const QString &id, bool enable)
     settingsPage().enableSettings(id, enable);
 }
 
-void LanguageClientSettings::toSettings(QSettings *settings,
+void LanguageClientSettings::toSettings(QtcSettings *settings,
                                         const QList<BaseSettings *> &languageClientSettings)
 {
     settings->beginGroup(settingsGroupKey);
     auto transform = [](const QList<BaseSettings *> &settings) {
         return Utils::transform(settings, [](const BaseSettings *setting) {
-            return QVariant(setting->toMap());
+            return variantFromStore(setting->toMap());
         });
     };
     auto isStdioSetting = Utils::equal(&BaseSettings::m_settingsTypeId,
@@ -755,18 +714,18 @@ bool StdIOSettings::isValid() const
     return BaseSettings::isValid() && !m_executable.isEmpty();
 }
 
-QVariantMap StdIOSettings::toMap() const
+Store StdIOSettings::toMap() const
 {
-    QVariantMap map = BaseSettings::toMap();
-    map.insert(executableKey, m_executable.toVariant());
+    Store map = BaseSettings::toMap();
+    map.insert(executableKey, m_executable.toSettings());
     map.insert(argumentsKey, m_arguments);
     return map;
 }
 
-void StdIOSettings::fromMap(const QVariantMap &map)
+void StdIOSettings::fromMap(const Store &map)
 {
     BaseSettings::fromMap(map);
-    m_executable = Utils::FilePath::fromVariant(map[executableKey]);
+    m_executable = Utils::FilePath::fromSettings(map[executableKey]);
     m_arguments = map[argumentsKey].toString();
 }
 
@@ -813,12 +772,11 @@ static QString startupBehaviorString(BaseSettings::StartBehavior behavior)
 {
     switch (behavior) {
     case BaseSettings::AlwaysOn:
-        return QCoreApplication::translate("LanguageClient::BaseSettings", "Always On");
+        return Tr::tr("Always On");
     case BaseSettings::RequiresFile:
-        return QCoreApplication::translate("LanguageClient::BaseSettings", "Requires an Open File");
+        return Tr::tr("Requires an Open File");
     case BaseSettings::RequiresProject:
-        return QCoreApplication::translate("LanguageClient::BaseSettings",
-                                           "Start Server per Project");
+        return Tr::tr("Start Server per Project");
     default:
         break;
     }
@@ -836,22 +794,24 @@ BaseSettingsWidget::BaseSettingsWidget(const BaseSettings *settings, QWidget *pa
     int row = 0;
     auto *mainLayout = new QGridLayout;
 
-    mainLayout->addWidget(new QLabel(tr("Name:")), row, 0);
+    mainLayout->addWidget(new QLabel(Tr::tr("Name:")), row, 0);
     mainLayout->addWidget(m_name, row, 1);
     auto chooser = new Utils::VariableChooser(this);
     chooser->addSupportedWidget(m_name);
 
-    mainLayout->addWidget(new QLabel(tr("Language:")), ++row, 0);
+    mainLayout->addWidget(new QLabel(Tr::tr("Language:")), ++row, 0);
     auto mimeLayout = new QHBoxLayout;
     mimeLayout->addWidget(m_mimeTypes);
     mimeLayout->addStretch();
-    auto addMimeTypeButton = new QPushButton(tr("Set MIME Types..."), this);
+    auto addMimeTypeButton = new QPushButton(Tr::tr("Set MIME Types..."), this);
     mimeLayout->addWidget(addMimeTypeButton);
     mainLayout->addLayout(mimeLayout, row, 1);
-    m_filePattern->setPlaceholderText(tr("File pattern"));
+    m_filePattern->setPlaceholderText(Tr::tr("File pattern"));
+    m_filePattern->setToolTip(
+        Tr::tr("List of file patterns.\nExample: *.cpp%1*.h").arg(filterSeparator));
     mainLayout->addWidget(m_filePattern, ++row, 1);
 
-    mainLayout->addWidget(new QLabel(tr("Startup behavior:")), ++row, 0);
+    mainLayout->addWidget(new QLabel(Tr::tr("Startup behavior:")), ++row, 0);
     for (int behavior = 0; behavior < BaseSettings::LastSentinel ; ++behavior)
         m_startupBehavior->addItem(startupBehaviorString(BaseSettings::StartBehavior(behavior)));
     m_startupBehavior->setCurrentIndex(settings->m_startBehavior);
@@ -861,7 +821,7 @@ BaseSettingsWidget::BaseSettingsWidget(const BaseSettings *settings, QWidget *pa
     connect(addMimeTypeButton, &QPushButton::pressed,
             this, &BaseSettingsWidget::showAddMimeTypeDialog);
 
-    mainLayout->addWidget(new QLabel(tr("Initialization options:")), ++row, 0);
+    mainLayout->addWidget(new QLabel(Tr::tr("Initialization options:")), ++row, 0);
     mainLayout->addWidget(m_initializationOptions, row, 1);
     chooser->addSupportedWidget(m_initializationOptions);
     m_initializationOptions->setValidationFunction([](Utils::FancyLineEdit *edit, QString *errorMessage) {
@@ -875,7 +835,7 @@ BaseSettingsWidget::BaseSettingsWidget(const BaseSettings *settings, QWidget *pa
 
         if (json.isNull()) {
             if (errorMessage)
-                *errorMessage = tr("Failed to parse JSON at %1: %2")
+                *errorMessage = Tr::tr("Failed to parse JSON at %1: %2")
                     .arg(parseInfo.offset)
                     .arg(parseInfo.errorString());
             return false;
@@ -883,7 +843,7 @@ BaseSettingsWidget::BaseSettingsWidget(const BaseSettings *settings, QWidget *pa
         return true;
     });
     m_initializationOptions->setText(settings->m_initializationOptions);
-    m_initializationOptions->setPlaceholderText(tr("Language server-specific JSON to pass via "
+    m_initializationOptions->setPlaceholderText(Tr::tr("Language server-specific JSON to pass via "
                                                    "\"initializationOptions\" field of \"initialize\" "
                                                    "request."));
 
@@ -949,12 +909,11 @@ public:
 
 class MimeTypeDialog : public QDialog
 {
-    Q_DECLARE_TR_FUNCTIONS(MimeTypeDialog)
 public:
     explicit MimeTypeDialog(const QStringList &selectedMimeTypes, QWidget *parent = nullptr)
         : QDialog(parent)
     {
-        setWindowTitle(tr("Select MIME Types"));
+        setWindowTitle(Tr::tr("Select MIME Types"));
         auto mainLayout = new QVBoxLayout;
         auto filter = new Utils::FancyLineEdit(this);
         filter->setFiltering(true);
@@ -965,7 +924,7 @@ public:
         mainLayout->addWidget(buttons);
         setLayout(mainLayout);
 
-        filter->setPlaceholderText(tr("Filter"));
+        filter->setPlaceholderText(Tr::tr("Filter"));
         connect(buttons, &QDialogButtonBox::accepted, this, &QDialog::accept);
         connect(buttons, &QDialogButtonBox::rejected, this, &QDialog::reject);
         auto proxy = new QSortFilterProxyModel(this);
@@ -1012,9 +971,9 @@ StdIOSettingsWidget::StdIOSettingsWidget(const StdIOSettings *settings, QWidget
     auto mainLayout = qobject_cast<QGridLayout *>(layout());
     QTC_ASSERT(mainLayout, return);
     const int baseRows = mainLayout->rowCount();
-    mainLayout->addWidget(new QLabel(tr("Executable:")), baseRows, 0);
+    mainLayout->addWidget(new QLabel(Tr::tr("Executable:")), baseRows, 0);
     mainLayout->addWidget(m_executable, baseRows, 1);
-    mainLayout->addWidget(new QLabel(tr("Arguments:")), baseRows + 1, 0);
+    mainLayout->addWidget(new QLabel(Tr::tr("Arguments:")), baseRows + 1, 0);
     m_executable->setExpectedKind(Utils::PathChooser::ExistingCommand);
     m_executable->setFilePath(settings->m_executable);
     mainLayout->addWidget(m_arguments, baseRows + 1, 1);
@@ -1070,18 +1029,28 @@ bool LanguageFilter::operator!=(const LanguageFilter &other) const
 TextEditor::BaseTextEditor *jsonEditor()
 {
     using namespace TextEditor;
-    BaseTextEditor *editor = PlainTextEditorFactory::createPlainTextEditor();
-    TextDocument *document = editor->textDocument();
-    TextEditorWidget *widget = editor->editorWidget();
+    using namespace Utils::Text;
+    BaseTextEditor *textEditor = nullptr;
+    for (Core::IEditorFactory *factory : Core::IEditorFactory::preferredEditorFactories("foo.json")) {
+        Core::IEditor *editor = factory->createEditor();
+        if (textEditor = qobject_cast<BaseTextEditor *>(editor); textEditor)
+            break;
+        delete editor;
+    }
+    QTC_ASSERT(textEditor, textEditor = PlainTextEditorFactory::createPlainTextEditor());
+    TextDocument *document = textEditor->textDocument();
+    TextEditorWidget *widget = textEditor->editorWidget();
     widget->configureGenericHighlighter(Utils::mimeTypeForName("application/json"));
     widget->setLineNumbersVisible(false);
-    widget->setMarksVisible(false);
     widget->setRevisionsVisible(false);
     widget->setCodeFoldingSupported(false);
     QObject::connect(document, &TextDocument::contentsChanged, widget, [document](){
         const Utils::Id jsonMarkId("LanguageClient.JsonTextMarkId");
-        qDeleteAll(
-            Utils::filtered(document->marks(), Utils::equal(&TextMark::category, jsonMarkId)));
+        const TextMarks marks = document->marks();
+        for (TextMark *mark : marks) {
+            if (mark->category().id == jsonMarkId)
+                delete mark;
+        }
         const QString content = document->plainText().trimmed();
         if (content.isEmpty())
             return;
@@ -1089,17 +1058,77 @@ TextEditor::BaseTextEditor *jsonEditor()
         QJsonDocument::fromJson(content.toUtf8(), &error);
         if (error.error == QJsonParseError::NoError)
             return;
-        const Utils::OptionalLineColumn lineColumn
-            = Utils::Text::convertPosition(document->document(), error.offset);
-        if (!lineColumn.has_value())
+        const Position pos = Position::fromPositionInDocument(document->document(), error.offset);
+        if (!pos.isValid())
             return;
-        auto mark = new TextMark(Utils::FilePath(), lineColumn->line, jsonMarkId);
+        auto mark = new TextMark(Utils::FilePath(),
+                                 pos.line,
+                                 {::LanguageClient::Tr::tr("JSON Error"), jsonMarkId});
         mark->setLineAnnotation(error.errorString());
         mark->setColor(Utils::Theme::CodeModel_Error_TextMarkColor);
         mark->setIcon(Utils::Icons::CODEMODEL_ERROR.icon());
         document->addMark(mark);
     });
-    return editor;
+    return textEditor;
+}
+
+constexpr const char projectSettingsId[] = "LanguageClient.ProjectSettings";
+
+ProjectSettings::ProjectSettings(ProjectExplorer::Project *project)
+    : m_project(project)
+{
+    m_json = m_project->namedSettings(projectSettingsId).toByteArray();
+}
+
+QJsonValue ProjectSettings::workspaceConfiguration() const
+{
+    const auto doc = QJsonDocument::fromJson(m_json);
+    if (doc.isObject())
+        return doc.object();
+    if (doc.isArray())
+        return doc.array();
+    return {};
+}
+
+QByteArray ProjectSettings::json() const
+{
+    return m_json;
+}
+
+void ProjectSettings::setJson(const QByteArray &json)
+{
+    const QJsonValue oldConfig = workspaceConfiguration();
+    m_json = json;
+    m_project->setNamedSettings(projectSettingsId, m_json);
+    const QJsonValue newConfig = workspaceConfiguration();
+    if (oldConfig != newConfig)
+        LanguageClientManager::updateWorkspaceConfiguration(m_project, newConfig);
+}
+
+ProjectSettingsWidget::ProjectSettingsWidget(ProjectExplorer::Project *project)
+    : m_settings(project)
+{
+    setUseGlobalSettingsCheckBoxVisible(false);
+    setGlobalSettingsId(Constants::LANGUAGECLIENT_SETTINGS_PAGE);
+    setExpanding(true);
+
+    TextEditor::BaseTextEditor *editor = jsonEditor();
+    editor->document()->setContents(m_settings.json());
+
+    auto layout = new QVBoxLayout;
+    setLayout(layout);
+    auto group = new QGroupBox(Tr::tr("Workspace Configuration"));
+    group->setLayout(new QVBoxLayout);
+    group->layout()->addWidget(new QLabel(Tr::tr(
+        "Additional JSON configuration sent to all running language servers for this project.\n"
+        "See the documentation of the specific language server for valid settings.")));
+    group->layout()->addWidget(editor->widget());
+    layout->addWidget(group);
+
+    connect(editor->editorWidget()->textDocument(),
+            &TextEditor::TextDocument::contentsChanged,
+            this,
+            [=]() { m_settings.setJson(editor->document()->contents()); });
 }
 
 } // namespace LanguageClient
