@@ -1,27 +1,5 @@
-/****************************************************************************
-**
-** Copyright (C) 2016 The Qt Company Ltd.
-** Contact: https://www.qt.io/licensing/
-**
-** This file is part of Qt Creator.
-**
-** Commercial License Usage
-** Licensees holding valid commercial Qt licenses may use this file in
-** accordance with the commercial license agreement provided with the
-** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and The Qt Company. For licensing terms
-** and conditions see https://www.qt.io/terms-conditions. For further
-** information use the contact form at https://www.qt.io/contact-us.
-**
-** GNU General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU
-** General Public License version 3 as published by the Free Software
-** Foundation with exceptions as appearing in the file LICENSE.GPL3-EXCEPT
-** included in the packaging of this file. Please review the following
-** information to ensure the GNU General Public License requirements will
-** be met: https://www.gnu.org/licenses/gpl-3.0.html.
-**
-****************************************************************************/
+// Copyright (C) 2016 The Qt Company Ltd.
+// SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only WITH Qt-GPL-exception-1.0
 
 #include "projectwindow.h"
 
@@ -31,12 +9,12 @@
 #include "kitoptionspage.h"
 #include "panelswidget.h"
 #include "project.h"
-#include "projectexplorer.h"
 #include "projectexplorerconstants.h"
+#include "projectexplorertr.h"
 #include "projectimporter.h"
+#include "projectmanager.h"
 #include "projectpanelfactory.h"
 #include "projectsettingswidget.h"
-#include "session.h"
 #include "target.h"
 #include "targetsettingspanel.h"
 
@@ -44,6 +22,7 @@
 #include <coreplugin/actionmanager/commandbutton.h>
 #include <coreplugin/coreconstants.h>
 #include <coreplugin/coreicons.h>
+#include <coreplugin/coreplugintr.h>
 #include <coreplugin/find/optionspopup.h>
 #include <coreplugin/findplaceholder.h>
 #include <coreplugin/icontext.h>
@@ -56,6 +35,7 @@
 #include <utils/hostosinfo.h>
 #include <utils/navigationtreeview.h>
 #include <utils/qtcassert.h>
+#include <utils/qtcsettings.h>
 #include <utils/styledbar.h>
 #include <utils/treemodel.h>
 #include <utils/utilsicons.h>
@@ -123,14 +103,14 @@ BuildSystemOutputWindow::BuildSystemOutputWindow()
     connect(&m_clear, &QAction::triggered, this, &OutputWindow::clear);
 
     m_filterActionRegexp.setCheckable(true);
-    m_filterActionRegexp.setText(ProjectWindow::tr("Use Regular Expressions"));
+    m_filterActionRegexp.setText(Tr::tr("Use Regular Expressions"));
     connect(&m_filterActionRegexp, &QAction::toggled, this, &BuildSystemOutputWindow::updateFilter);
     Core::ActionManager::registerAction(&m_filterActionRegexp,
                                         kRegExpActionId,
                                         Context(Constants::C_PROJECTEXPLORER));
 
     m_filterActionCaseSensitive.setCheckable(true);
-    m_filterActionCaseSensitive.setText(ProjectWindow::tr("Case Sensitive"));
+    m_filterActionCaseSensitive.setText(Tr::tr("Case Sensitive"));
     connect(&m_filterActionCaseSensitive,
             &QAction::toggled,
             this,
@@ -140,7 +120,7 @@ BuildSystemOutputWindow::BuildSystemOutputWindow()
                                         Context(Constants::C_PROJECTEXPLORER));
 
     m_invertFilterAction.setCheckable(true);
-    m_invertFilterAction.setText(ProjectWindow::tr("Show Non-matching Lines"));
+    m_invertFilterAction.setText(Tr::tr("Show Non-matching Lines"));
     connect(&m_invertFilterAction, &QAction::toggled, this, &BuildSystemOutputWindow::updateFilter);
     Core::ActionManager::registerAction(&m_invertFilterAction,
                                         kInvertActionId,
@@ -158,7 +138,7 @@ BuildSystemOutputWindow::BuildSystemOutputWindow()
                                   Core::Constants::ZOOM_IN,
                                   Context(kBuildSystemOutputContext));
 
-    m_zoomOut.setIcon(Utils::Icons::MINUS.icon());
+    m_zoomOut.setIcon(Utils::Icons::MINUS_TOOLBAR.icon());
     connect(&m_zoomOut, &QAction::triggered, this, [this] { zoomOut(); });
     ActionManager::registerAction(&m_zoomOut,
                                   Core::Constants::ZOOM_OUT,
@@ -266,8 +246,7 @@ QVariant MiscSettingsPanelItem::data(int column, int role) const
 
     if (role == ActiveItemRole)  // We are the active one.
         return QVariant::fromValue<TreeItem *>(const_cast<MiscSettingsPanelItem *>(this));
-
-    return QVariant();
+    return {};
 }
 
 Qt::ItemFlags MiscSettingsPanelItem::flags(int column) const
@@ -313,14 +292,14 @@ public:
     {
         switch (role) {
         case Qt::DisplayRole:
-            return ProjectWindow::tr("Project Settings");
+            return Tr::tr("Project Settings");
 
         case PanelWidgetRole:
         case ActiveItemRole:
             if (0 <= m_currentPanelIndex && m_currentPanelIndex < childCount())
                 return childAt(m_currentPanelIndex)->data(column, role);
         }
-        return QVariant();
+        return {};
     }
 
     bool setData(int column, const QVariant &data, int role) override
@@ -358,7 +337,7 @@ public:
         : m_project(project), m_changeListener(changeListener)
     {
         QTC_ASSERT(m_project, return);
-        QString display = ProjectWindow::tr("Build & Run");
+        QString display = Tr::tr("Build & Run");
         appendChild(m_targetsItem = new TargetGroupItem(display, project));
         appendChild(m_miscItem = new MiscSettingsGroupItem(project));
     }
@@ -372,7 +351,7 @@ public:
 
         case Qt::FontRole: {
             QFont font;
-            font.setBold(m_project == SessionManager::startupProject());
+            font.setBold(m_project == ProjectManager::startupProject());
             return font;
         }
 
@@ -383,7 +362,7 @@ public:
             if (m_currentChildIndex == 1)
                 return m_miscItem->data(column, role);
         }
-        return QVariant();
+        return {};
     }
 
     bool setData(int column, const QVariant &dat, int role) override
@@ -412,7 +391,7 @@ public:
 
         if (role == ItemActivatedDirectlyRole) {
             // Someone selected the project using the combobox or similar.
-            SessionManager::setStartupProject(m_project);
+            ProjectManager::setStartupProject(m_project);
             m_currentChildIndex = 0; // Use some Target page by defaults
             m_targetsItem->setData(column, dat, ItemActivatedFromAboveRole); // And propagate downwards.
             announceChange();
@@ -542,7 +521,7 @@ public:
     ProjectWindowPrivate(ProjectWindow *parent)
         : q(parent)
     {
-        m_projectsModel.setHeader({ProjectWindow::tr("Projects")});
+        m_projectsModel.setHeader({Tr::tr("Projects")});
 
         m_selectorTree = new SelectorTree;
         m_selectorTree->setModel(&m_projectsModel);
@@ -555,7 +534,7 @@ public:
 
         m_projectSelection = new QComboBox;
         m_projectSelection->setModel(&m_comboBoxModel);
-        connect(m_projectSelection, QOverload<int>::of(&QComboBox::activated),
+        connect(m_projectSelection, &QComboBox::activated,
                 this, &ProjectWindowPrivate::projectSelected, Qt::QueuedConnection);
 
         const auto switchProjectAction = new QAction(this);
@@ -566,22 +545,22 @@ public:
                 m_projectSelection->showPopup();
         });
 
-        SessionManager *sessionManager = SessionManager::instance();
-        connect(sessionManager, &SessionManager::projectAdded,
+        ProjectManager *sessionManager = ProjectManager::instance();
+        connect(sessionManager, &ProjectManager::projectAdded,
                 this, &ProjectWindowPrivate::registerProject);
-        connect(sessionManager, &SessionManager::aboutToRemoveProject,
+        connect(sessionManager, &ProjectManager::aboutToRemoveProject,
                 this, &ProjectWindowPrivate::deregisterProject);
-        connect(sessionManager, &SessionManager::startupProjectChanged,
+        connect(sessionManager, &ProjectManager::startupProjectChanged,
                 this, &ProjectWindowPrivate::startupProjectChanged);
 
-        m_importBuild = new QPushButton(ProjectWindow::tr("Import Existing Build..."));
+        m_importBuild = new QPushButton(Tr::tr("Import Existing Build..."));
         connect(m_importBuild, &QPushButton::clicked,
                 this, &ProjectWindowPrivate::handleImportBuild);
-        connect(sessionManager, &SessionManager::startupProjectChanged, this, [this](Project *project) {
+        connect(sessionManager, &ProjectManager::startupProjectChanged, this, [this](Project *project) {
             m_importBuild->setEnabled(project && project->projectImporter());
         });
 
-        m_manageKits = new QPushButton(ProjectWindow::tr("Manage Kits..."));
+        m_manageKits = new QPushButton(Tr::tr("Manage Kits..."));
         connect(m_manageKits, &QPushButton::clicked,
                 this, &ProjectWindowPrivate::handleManageKits);
 
@@ -590,13 +569,10 @@ public:
 
         auto selectorView = new QWidget; // Black blob + Combobox + Project tree below.
         selectorView->setObjectName("ProjectSelector"); // Needed for dock widget state saving
-        selectorView->setWindowTitle(ProjectWindow::tr("Project Selector"));
+        selectorView->setWindowTitle(Tr::tr("Project Selector"));
         selectorView->setAutoFillBackground(true);
-        selectorView->setContextMenuPolicy(Qt::CustomContextMenu);
-        connect(selectorView, &QWidget::customContextMenuRequested,
-                this, &ProjectWindowPrivate::openContextMenu);
 
-        auto activeLabel = new QLabel(ProjectWindow::tr("Active Project"));
+        auto activeLabel = new QLabel(Tr::tr("Active Project"));
         QFont font = activeLabel->font();
         font.setBold(true);
         font.setPointSizeF(font.pointSizeF() * 1.2);
@@ -606,8 +582,13 @@ public:
         innerLayout->setSpacing(10);
         innerLayout->setContentsMargins(PanelsWidget::PanelVMargin, innerLayout->spacing(),
                                         PanelsWidget::PanelVMargin, 0);
-        innerLayout->addWidget(m_manageKits);
-        innerLayout->addSpacerItem(new QSpacerItem(10, 30, QSizePolicy::Maximum, QSizePolicy::Maximum));
+
+        QStringList list = Core::ICore::settings()->value("HideOptionCategories").toStringList();
+        if (!list.contains("Kits")) {
+            innerLayout->addWidget(m_manageKits);
+            innerLayout->addSpacerItem(new QSpacerItem(10, 30, QSizePolicy::Maximum, QSizePolicy::Maximum));
+        }
+
         innerLayout->addWidget(activeLabel);
         innerLayout->addWidget(m_projectSelection);
         innerLayout->addWidget(m_importBuild);
@@ -627,7 +608,7 @@ public:
         // especially the find tool bar (resulting in wrong label color)
         output->setBackgroundRole(QPalette::Window);
         output->setObjectName("BuildSystemOutput");
-        output->setWindowTitle(ProjectWindow::tr("Build System Output"));
+        output->setWindowTitle(Tr::tr("Build System Output"));
         auto outputLayout = new QVBoxLayout;
         output->setLayout(outputLayout);
         outputLayout->setContentsMargins(0, 0, 0, 0);
@@ -635,8 +616,27 @@ public:
         outputLayout->addWidget(m_buildSystemOutput->toolBar());
         outputLayout->addWidget(m_buildSystemOutput);
         outputLayout->addWidget(new FindToolBarPlaceHolder(m_buildSystemOutput));
-        auto outputDock = q->addDockForWidget(output, true);
-        q->addDockWidget(Qt::RightDockWidgetArea, outputDock);
+        m_outputDock = q->addDockForWidget(output, true);
+        q->addDockWidget(Qt::RightDockWidgetArea, m_outputDock);
+
+        m_toggleRightSidebarAction.setCheckable(true);
+        m_toggleRightSidebarAction.setChecked(true);
+        const auto toolTipText = [](bool checked) {
+            return checked ? ::Core::Tr::tr(Core::Constants::TR_HIDE_RIGHT_SIDEBAR)
+                           : ::Core::Tr::tr(Core::Constants::TR_SHOW_RIGHT_SIDEBAR);
+        };
+        m_toggleRightSidebarAction.setText(toolTipText(false)); // always "Show Right Sidebar"
+        m_toggleRightSidebarAction.setToolTip(toolTipText(m_toggleRightSidebarAction.isChecked()));
+        ActionManager::registerAction(&m_toggleRightSidebarAction,
+                                      Core::Constants::TOGGLE_RIGHT_SIDEBAR,
+                                      Context(Constants::C_PROJECTEXPLORER));
+        connect(&m_toggleRightSidebarAction,
+                &QAction::toggled,
+                this,
+                [this, toolTipText](bool checked) {
+                    m_toggleRightSidebarAction.setToolTip(toolTipText(checked));
+                    m_outputDock->setVisible(checked);
+                });
     }
 
     void updatePanel()
@@ -673,7 +673,7 @@ public:
     void projectSelected(int index)
     {
         Project *project = m_comboBoxModel.rootItem()->childAt(index)->m_projectItem->project();
-        SessionManager::setStartupProject(project);
+        ProjectManager::setStartupProject(project);
     }
 
     ComboBoxItem *itemForProject(Project *project) const
@@ -727,9 +727,9 @@ public:
         if (!menu.actions().isEmpty())
             menu.addSeparator();
 
-        QAction *importBuild = menu.addAction(ProjectWindow::tr("Import Existing Build..."));
+        QAction *importBuild = menu.addAction(Tr::tr("Import Existing Build..."));
         importBuild->setEnabled(project && project->projectImporter());
-        QAction *manageKits = menu.addAction(ProjectWindow::tr("Manage Kits..."));
+        QAction *manageKits = menu.addAction(Tr::tr("Manage Kits..."));
 
         QAction *act = menu.exec(m_selectorTree->mapToGlobal(pos));
 
@@ -741,9 +741,17 @@ public:
 
     void handleManageKits()
     {
-        if (ProjectItem *projectItem = m_projectsModel.rootItem()->childAt(0)) {
-            if (auto kitPage = KitOptionsPage::instance())
-                kitPage->showKit(KitManager::kit(Id::fromSetting(projectItem->data(0, KitIdRole))));
+        const QModelIndexList selected = m_selectorTree->selectionModel()->selectedIndexes();
+        if (!selected.isEmpty()) {
+            TreeItem *treeItem = m_projectsModel.itemForIndex(selected.front());
+            while (treeItem) {
+                const Id kitId = Id::fromSetting(treeItem->data(0, KitIdRole));
+                if (kitId.isValid()) {
+                    setSelectectKitId(kitId);
+                    break;
+                }
+                treeItem = treeItem->parent();
+            }
         }
         ICore::showOptionsDialog(Constants::KITS_SETTINGS_PAGE_ID);
     }
@@ -756,7 +764,7 @@ public:
         QTC_ASSERT(projectImporter, return);
 
         FilePath importDir =
-                FileUtils::getExistingDirectory(nullptr, ProjectWindow::tr("Import Directory"),
+                FileUtils::getExistingDirectory(nullptr, Tr::tr("Import Directory"),
                                                 project->projectDirectory());
 
         Target *lastTarget = nullptr;
@@ -776,8 +784,8 @@ public:
             }
         }
         if (lastTarget && lastBc) {
-            SessionManager::setActiveBuildConfiguration(lastTarget, lastBc, SetActive::Cascade);
-            SessionManager::setActiveTarget(project, lastTarget, SetActive::Cascade);
+            lastTarget->setActiveBuildConfiguration(lastBc, SetActive::Cascade);
+            project->setActiveTarget(lastTarget, SetActive::Cascade);
         }
     }
 
@@ -805,6 +813,8 @@ public:
     SelectorTree *m_selectorTree;
     QPushButton *m_importBuild;
     QPushButton *m_manageKits;
+    QAction m_toggleRightSidebarAction;
+    QDockWidget *m_outputDock;
     BuildSystemOutputWindow *m_buildSystemOutput;
 };
 
@@ -852,7 +862,7 @@ void ProjectWindow::savePersistentSettings() const
 {
     if (!centralWidget())
         return;
-    QSettings * const settings = ICore::settings();
+    QtcSettings * const settings = ICore::settings();
     settings->beginGroup(PROJECT_WINDOW_KEY);
     saveSettings(settings);
     settings->endGroup();
@@ -862,10 +872,11 @@ void ProjectWindow::loadPersistentSettings()
 {
     if (!centralWidget())
         return;
-    QSettings * const settings = ICore::settings();
+    QtcSettings * const settings = ICore::settings();
     settings->beginGroup(PROJECT_WINDOW_KEY);
     restoreSettings(settings);
     settings->endGroup();
+    d->m_toggleRightSidebarAction.setChecked(d->m_outputDock->isVisible());
 }
 
 QSize SelectorDelegate::sizeHint(const QStyleOptionViewItem &option, const QModelIndex &index) const
