@@ -1,27 +1,5 @@
-/****************************************************************************
-**
-** Copyright (C) 2016 The Qt Company Ltd.
-** Contact: https://www.qt.io/licensing/
-**
-** This file is part of Qt Creator.
-**
-** Commercial License Usage
-** Licensees holding valid commercial Qt licenses may use this file in
-** accordance with the commercial license agreement provided with the
-** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and The Qt Company. For licensing terms
-** and conditions see https://www.qt.io/terms-conditions. For further
-** information use the contact form at https://www.qt.io/contact-us.
-**
-** GNU General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU
-** General Public License version 3 as published by the Free Software
-** Foundation with exceptions as appearing in the file LICENSE.GPL3-EXCEPT
-** included in the packaging of this file. Please review the following
-** information to ensure the GNU General Public License requirements will
-** be met: https://www.gnu.org/licenses/gpl-3.0.html.
-**
-****************************************************************************/
+// Copyright (C) 2016 The Qt Company Ltd.
+// SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only WITH Qt-GPL-exception-1.0
 
 #pragma once
 
@@ -29,8 +7,6 @@
 
 #include "toolchain.h"
 
-#include <coreplugin/id.h>
-
 #include <QList>
 #include <QObject>
 #include <QSet>
@@ -63,28 +39,32 @@ public:
     static ToolChainManager *instance();
     ~ToolChainManager() override;
 
-    static QList<ToolChain *> toolChains(const ToolChain::Predicate &predicate = ToolChain::Predicate());
+    static const Toolchains &toolchains();
+    static Toolchains toolchains(const ToolChain::Predicate &predicate);
+
     static ToolChain *toolChain(const ToolChain::Predicate &predicate);
     static QList<ToolChain *> findToolChains(const Abi &abi);
     static ToolChain *findToolChain(const QByteArray &id);
 
-    static Utils::FilePath defaultDebugger(const Abi &abi);
-
     static bool isLoaded();
 
     static bool registerToolChain(ToolChain *tc);
     static void deregisterToolChain(ToolChain *tc);
 
-    static QSet<Core::Id> allLanguages();
-    static bool registerLanguage(const Core::Id &language, const QString &displayName);
-    static QString displayNameOfLanguageId(const Core::Id &id);
-    static bool isLanguageSupported(const Core::Id &id);
+    static QList<Utils::Id> allLanguages();
+    static bool registerLanguage(const Utils::Id &language, const QString &displayName);
+    static QString displayNameOfLanguageId(const Utils::Id &id);
+    static bool isLanguageSupported(const Utils::Id &id);
 
     static void aboutToShutdown();
 
     static ToolchainDetectionSettings detectionSettings();
     static void setDetectionSettings(const ToolchainDetectionSettings &settings);
 
+    static void resetBadToolchains();
+    static bool isBadToolchain(const Utils::FilePath &toolchain);
+    static void addBadToolchain(const Utils::FilePath &toolchain);
+
     void saveToolChains();
 
 signals:
@@ -101,12 +81,9 @@ signals:
 private:
     explicit ToolChainManager(QObject *parent = nullptr);
 
-    // Make sure the this is only called after all
-    // Tool chain Factories are registered!
+    // Make sure the this is only called after all toolchain factories are registered!
     static void restoreToolChains();
 
-    static QList<ToolChain *> readSystemFileToolChains();
-
     static void notifyAboutUpdate(ToolChain *);
 
     friend class ProjectExplorerPlugin; // for constructor
