@@ -1,37 +1,17 @@
-/****************************************************************************
-**
-** Copyright (C) 2016 The Qt Company Ltd.
-** Contact: https://www.qt.io/licensing/
-**
-** This file is part of Qt Creator.
-**
-** Commercial License Usage
-** Licensees holding valid commercial Qt licenses may use this file in
-** accordance with the commercial license agreement provided with the
-** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and The Qt Company. For licensing terms
-** and conditions see https://www.qt.io/terms-conditions. For further
-** information use the contact form at https://www.qt.io/contact-us.
-**
-** GNU General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU
-** General Public License version 3 as published by the Free Software
-** Foundation with exceptions as appearing in the file LICENSE.GPL3-EXCEPT
-** included in the packaging of this file. Please review the following
-** information to ensure the GNU General Public License requirements will
-** be met: https://www.gnu.org/licenses/gpl-3.0.html.
-**
-****************************************************************************/
+// Copyright (C) 2016 The Qt Company Ltd.
+// SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only WITH Qt-GPL-exception-1.0
 
-#include "qmlprofilertraceview.h"
-#include "qmlprofilertool.h"
-#include "qmlprofilerstatemanager.h"
+#include "qmlprofileranimationsmodel.h"
 #include "qmlprofilermodelmanager.h"
 #include "qmlprofilernotesmodel.h"
-#include "qmlprofileranimationsmodel.h"
-#include "qmlprofilerrangemodel.h"
 #include "qmlprofilerplugin.h"
+#include "qmlprofilerrangemodel.h"
+#include "qmlprofilerstatemanager.h"
+#include "qmlprofilertool.h"
+#include "qmlprofilertr.h"
+#include "qmlprofilertraceview.h"
 
+#include "quick3dmodel.h"
 #include "inputeventsmodel.h"
 #include "pixmapcachemodel.h"
 #include "debugmessagesmodel.h"
@@ -64,14 +44,13 @@
 #include <QScrollBar>
 #include <QSlider>
 #include <QMenu>
+#include <QQmlEngine>
 #include <QQuickItem>
 #include <QQuickWidget>
 #include <QApplication>
-#include <QRegExp>
+#include <QRegularExpression>
 #include <QTextCursor>
 
-#include <math.h>
-
 namespace QmlProfiler {
 namespace Internal {
 
@@ -92,7 +71,7 @@ QmlProfilerTraceView::QmlProfilerTraceView(QWidget *parent, QmlProfilerViewManag
                                            QmlProfilerModelManager *modelManager)
     : QWidget(parent), d(new QmlProfilerTraceViewPrivate(this))
 {
-    setWindowTitle(tr("Timeline"));
+    setWindowTitle(Tr::tr("Timeline"));
     setObjectName("QmlProfiler.Timeline.Dock");
 
     d->m_zoomControl = new Timeline::TimelineZoomControl(this);
@@ -120,23 +99,16 @@ QmlProfilerTraceView::QmlProfilerTraceView(QWidget *parent, QmlProfilerViewManag
         }
     });
 
-    QVBoxLayout *groupLayout = new QVBoxLayout;
+    auto groupLayout = new QVBoxLayout;
     groupLayout->setContentsMargins(0, 0, 0, 0);
     groupLayout->setSpacing(0);
 
-    qmlRegisterType<Timeline::TimelineRenderer>("TimelineRenderer", 1, 0, "TimelineRenderer");
-    qmlRegisterType<Timeline::TimelineOverviewRenderer>("TimelineOverviewRenderer", 1, 0,
-                                                        "TimelineOverviewRenderer");
-    qmlRegisterType<Timeline::TimelineZoomControl>();
-    qmlRegisterType<Timeline::TimelineModel>();
-    qmlRegisterType<Timeline::TimelineNotesModel>();
-
     d->m_mainView = new QQuickWidget(this);
     d->m_mainView->setResizeMode(QQuickWidget::SizeRootObjectToView);
     d->m_mainView->setSizePolicy(QSizePolicy::Expanding, QSizePolicy::Expanding);
     setFocusProxy(d->m_mainView);
 
-    Aggregation::Aggregate *agg = new Aggregation::Aggregate;
+    auto agg = new Aggregation::Aggregate;
     agg->add(d->m_mainView);
     agg->add(new TraceViewFindSupport(this, modelManager));
 
@@ -145,7 +117,8 @@ QmlProfilerTraceView::QmlProfilerTraceView(QWidget *parent, QmlProfilerViewManag
     setLayout(groupLayout);
 
     d->m_viewContainer = container;
-    d->m_modelProxy = new Timeline::TimelineModelAggregator(modelManager->notesModel(), this);
+    d->m_modelProxy = new Timeline::TimelineModelAggregator(this);
+    d->m_modelProxy->setNotes(modelManager->notesModel());
     d->m_modelManager = modelManager;
 
     QVariantList models;
@@ -154,6 +127,7 @@ QmlProfilerTraceView::QmlProfilerTraceView(QWidget *parent, QmlProfilerViewManag
     models.append(QVariant::fromValue(new MemoryUsageModel(modelManager, d->m_modelProxy)));
     models.append(QVariant::fromValue(new InputEventsModel(modelManager, d->m_modelProxy)));
     models.append(QVariant::fromValue(new DebugMessagesModel(modelManager, d->m_modelProxy)));
+    models.append(QVariant::fromValue(new Quick3DModel(modelManager, d->m_modelProxy)));
     models.append(QVariant::fromValue(new QmlProfilerAnimationsModel(modelManager,
                                                                      d->m_modelProxy)));
     for (int i = 0; i < MaximumRangeType; ++i) {
@@ -165,14 +139,14 @@ QmlProfilerTraceView::QmlProfilerTraceView(QWidget *parent, QmlProfilerViewManag
     // Minimum height: 5 rows of 20 pixels + scrollbar of 50 pixels + 20 pixels margin
     setMinimumHeight(170);
 
+    d->m_mainView->engine()->addImportPath(":/qt/qml/");
     Timeline::TimelineTheme::setupTheme(d->m_mainView->engine());
-    Timeline::TimeFormatter::setupTimeFormatter();
 
     d->m_mainView->rootContext()->setContextProperty(QLatin1String("timelineModelAggregator"),
                                                      d->m_modelProxy);
     d->m_mainView->rootContext()->setContextProperty(QLatin1String("zoomControl"),
                                                      d->m_zoomControl);
-    d->m_mainView->setSource(QUrl(QLatin1String("qrc:/tracing/MainView.qml")));
+    d->m_mainView->setSource(QUrl(QLatin1String("qrc:/qt/qml/QtCreator/Tracing/MainView.qml")));
 
     connect(d->m_modelProxy, &Timeline::TimelineModelAggregator::updateCursorPosition,
             this, &QmlProfilerTraceView::updateCursorPosition);
@@ -265,17 +239,17 @@ void QmlProfilerTraceView::showContextMenu(QPoint position)
     menu.addActions(QmlProfilerTool::profilerContextMenuActions());
     menu.addSeparator();
 
-    QAction *getLocalStatsAction = menu.addAction(tr("Analyze Current Range"));
+    QAction *getLocalStatsAction = menu.addAction(Tr::tr("Analyze Current Range"));
     if (!hasValidSelection())
         getLocalStatsAction->setEnabled(false);
 
-    QAction *getGlobalStatsAction = menu.addAction(tr("Analyze Full Range"));
+    QAction *getGlobalStatsAction = menu.addAction(Tr::tr("Analyze Full Range"));
     if (!d->m_modelManager->isRestrictedToRange())
         getGlobalStatsAction->setEnabled(false);
 
     if (d->m_zoomControl->traceDuration() > 0) {
         menu.addSeparator();
-        viewAllAction = menu.addAction(tr("Reset Zoom"));
+        viewAllAction = menu.addAction(Tr::tr("Reset Zoom"));
     }
 
     QAction *selectedAction = menu.exec(position);
@@ -295,8 +269,9 @@ void QmlProfilerTraceView::showContextMenu(QPoint position)
 
 bool QmlProfilerTraceView::isUsable() const
 {
-    return d->m_mainView->quickWindow()->rendererInterface()->graphicsApi()
-            == QSGRendererInterface::OpenGL;
+    const QSGRendererInterface::GraphicsApi api =
+            d->m_mainView->quickWindow()->rendererInterface()->graphicsApi();
+    return QSGRendererInterface::isApiRhiBased(api);
 }
 
 bool QmlProfilerTraceView::isSuspended() const
@@ -323,10 +298,10 @@ bool TraceViewFindSupport::supportsReplace() const
     return false;
 }
 
-Core::FindFlags TraceViewFindSupport::supportedFindFlags() const
+Utils::FindFlags TraceViewFindSupport::supportedFindFlags() const
 {
-    return Core::FindBackward | Core::FindCaseSensitively | Core::FindRegularExpression
-            | Core::FindWholeWords;
+    return Utils::FindBackward | Utils::FindCaseSensitively | Utils::FindRegularExpression
+            | Utils::FindWholeWords;
 }
 
 void TraceViewFindSupport::resetIncrementalSearch()
@@ -350,7 +325,7 @@ QString TraceViewFindSupport::completedFindString() const
 }
 
 Core::IFindSupport::Result TraceViewFindSupport::findIncremental(const QString &txt,
-                                                                 Core::FindFlags findFlags)
+                                                                 Utils::FindFlags findFlags)
 {
     if (m_incrementalStartPos < 0)
         m_incrementalStartPos = qMax(m_currentPosition, 0);
@@ -364,9 +339,9 @@ Core::IFindSupport::Result TraceViewFindSupport::findIncremental(const QString &
 }
 
 Core::IFindSupport::Result TraceViewFindSupport::findStep(const QString &txt,
-                                                          Core::FindFlags findFlags)
+                                                          Utils::FindFlags findFlags)
 {
-    int start = (findFlags & Core::FindBackward) ? m_currentPosition : m_currentPosition + 1;
+    int start = (findFlags & Utils::FindBackward) ? m_currentPosition : m_currentPosition + 1;
     bool wrapped;
     bool found = find(txt, findFlags, start, &wrapped);
     if (wrapped)
@@ -380,14 +355,14 @@ Core::IFindSupport::Result TraceViewFindSupport::findStep(const QString &txt,
 
 // "start" is the model index that is searched first in a forward search, i.e. as if the
 // "cursor" were between start-1 and start
-bool TraceViewFindSupport::find(const QString &txt, Core::FindFlags findFlags, int start,
+bool TraceViewFindSupport::find(const QString &txt, Utils::FindFlags findFlags, int start,
                                 bool *wrapped)
 {
     if (wrapped)
         *wrapped = false;
     if (!findOne(txt, findFlags, start)) {
         int secondStart;
-        if (findFlags & Core::FindBackward)
+        if (findFlags & Utils::FindBackward)
             secondStart = m_modelManager->notesModel()->count();
         else
             secondStart = 0;
@@ -401,18 +376,19 @@ bool TraceViewFindSupport::find(const QString &txt, Core::FindFlags findFlags, i
 
 // "start" is the model index that is searched first in a forward search, i.e. as if the
 // "cursor" were between start-1 and start
-bool TraceViewFindSupport::findOne(const QString &txt, Core::FindFlags findFlags, int start)
+bool TraceViewFindSupport::findOne(const QString &txt, Utils::FindFlags findFlags, int start)
 {
-    bool caseSensitiveSearch = (findFlags & Core::FindCaseSensitively);
-    QRegExp regexp(txt);
-    regexp.setPatternSyntax((findFlags & Core::FindRegularExpression) ? QRegExp::RegExp : QRegExp::FixedString);
-    regexp.setCaseSensitivity(caseSensitiveSearch ? Qt::CaseSensitive : Qt::CaseInsensitive);
+    bool caseSensitiveSearch = (findFlags & Utils::FindCaseSensitively);
+    bool regexSearch = (findFlags & Utils::FindRegularExpression);
+    QRegularExpression regexp(regexSearch ? txt : QRegularExpression::escape(txt),
+                              caseSensitiveSearch ? QRegularExpression::NoPatternOption
+                                                  : QRegularExpression::CaseInsensitiveOption);
     QTextDocument::FindFlags flags;
     if (caseSensitiveSearch)
         flags |= QTextDocument::FindCaseSensitively;
-    if (findFlags & Core::FindWholeWords)
+    if (findFlags & Utils::FindWholeWords)
         flags |= QTextDocument::FindWholeWords;
-    bool forwardSearch = !(findFlags & Core::FindBackward);
+    bool forwardSearch = !(findFlags & Utils::FindBackward);
     int increment = forwardSearch ? +1 : -1;
     int current = forwardSearch ? start : start - 1;
     Timeline::TimelineNotesModel *model = m_modelManager->notesModel();
