@@ -1,27 +1,5 @@
-/****************************************************************************
-**
-** Copyright (C) 2016 The Qt Company Ltd.
-** Contact: https://www.qt.io/licensing/
-**
-** This file is part of Qt Creator.
-**
-** Commercial License Usage
-** Licensees holding valid commercial Qt licenses may use this file in
-** accordance with the commercial license agreement provided with the
-** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and The Qt Company. For licensing terms
-** and conditions see https://www.qt.io/terms-conditions. For further
-** information use the contact form at https://www.qt.io/contact-us.
-**
-** GNU General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU
-** General Public License version 3 as published by the Free Software
-** Foundation with exceptions as appearing in the file LICENSE.GPL3-EXCEPT
-** included in the packaging of this file. Please review the following
-** information to ensure the GNU General Public License requirements will
-** be met: https://www.gnu.org/licenses/gpl-3.0.html.
-**
-****************************************************************************/
+// Copyright (C) 2016 The Qt Company Ltd.
+// SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only WITH Qt-GPL-exception-1.0
 
 #include "syntaxhighlighter.h"
 #include "textdocument.h"
@@ -34,20 +12,29 @@
 
 #include <QTextDocument>
 #include <QPointer>
-#include <qtimer.h>
 
 #include <cmath>
 
 namespace TextEditor {
 
+enum HighlighterTypeProperty
+{
+    SyntaxHighlight = QTextFormat::UserProperty + 1,
+    SemanticHighlight = QTextFormat::UserProperty + 2
+};
+
 class SyntaxHighlighterPrivate
 {
     SyntaxHighlighter *q_ptr = nullptr;
     Q_DECLARE_PUBLIC(SyntaxHighlighter)
 public:
     SyntaxHighlighterPrivate()
+        : SyntaxHighlighterPrivate(TextEditorSettings::fontSettings())
+    { }
+
+    SyntaxHighlighterPrivate(const FontSettings &fontSettings)
     {
-        updateFormats(TextEditorSettings::fontSettings());
+        updateFormats(fontSettings);
     }
 
     QPointer<QTextDocument> doc;
@@ -99,6 +86,16 @@ void SyntaxHighlighter::delayedRehighlight()
     rehighlight();
 }
 
+#ifdef WITH_TESTS
+SyntaxHighlighter::SyntaxHighlighter(QTextDocument *parent, const FontSettings &fontsettings)
+    : QObject(parent), d_ptr(new SyntaxHighlighterPrivate(fontsettings))
+{
+    d_ptr->q_ptr = this;
+    if (parent)
+        setDocument(parent);
+}
+#endif
+
 void SyntaxHighlighterPrivate::applyFormatChanges(int from, int charsRemoved, int charsAdded)
 {
     bool formatsChanged = false;
@@ -107,9 +104,9 @@ void SyntaxHighlighterPrivate::applyFormatChanges(int from, int charsRemoved, in
 
     QVector<QTextLayout::FormatRange> ranges;
     QVector<QTextLayout::FormatRange> oldRanges;
-    std::tie(ranges, oldRanges)
+    std::tie(oldRanges, ranges)
         = Utils::partition(layout->formats(), [](const QTextLayout::FormatRange &range) {
-              return range.format.property(QTextFormat::UserProperty).toBool();
+              return range.format.property(SyntaxHighlight).toBool();
           });
 
     if (currentBlock.contains(from)) {
@@ -138,8 +135,23 @@ void SyntaxHighlighterPrivate::applyFormatChanges(int from, int charsRemoved, in
         while (i < formatChanges.count() && formatChanges.at(i) == r.format)
             ++i;
 
+        r.format.setProperty(SyntaxHighlight, true);
         r.length = i - r.start;
 
+        const QString preeditText = currentBlock.layout()->preeditAreaText();
+        if (!preeditText.isEmpty()) {
+            const int preeditPosition = currentBlock.layout()->preeditAreaPosition();
+            if (r.start >= preeditPosition) {
+                r.start += preeditText.length();
+            } else if (r.start + r.length > preeditPosition) {
+                QTextLayout::FormatRange beforePreeditRange = r;
+                r.start = preeditPosition + preeditText.length();
+                r.length = r.length - (r.start - preeditPosition);
+                beforePreeditRange.length = preeditPosition - beforePreeditRange.start;
+                newRanges << beforePreeditRange;
+            }
+        }
+
         newRanges << r;
     }
 
@@ -310,7 +322,8 @@ void SyntaxHighlighter::setDocument(QTextDocument *doc)
         if (!d->noAutomaticHighlighting) {
             connect(d->doc, &QTextDocument::contentsChange, this, &SyntaxHighlighter::reformatBlocks);
             d->rehighlightPending = true;
-            QTimer::singleShot(0, this, &SyntaxHighlighter::delayedRehighlight);
+            QMetaObject::invokeMethod(this, &SyntaxHighlighter::delayedRehighlight,
+                                      Qt::QueuedConnection);
         }
         d->foldValidator.setup(qobject_cast<TextDocumentLayout *>(doc->documentLayout()));
     }
@@ -499,9 +512,7 @@ void SyntaxHighlighter::formatSpaces(const QString &text, int start, int count)
 void SyntaxHighlighter::setFormatWithSpaces(const QString &text, int start, int count,
                                             const QTextCharFormat &format)
 {
-    Q_D(const SyntaxHighlighter);
-    QTextCharFormat visualSpaceFormat = d->whitespaceFormat;
-    visualSpaceFormat.setBackground(format.background());
+    const QTextCharFormat visualSpaceFormat = whitespacified(format);
 
     const int end = std::min(start + count, int(text.length()));
     int index = start;
@@ -665,6 +676,24 @@ void SyntaxHighlighter::setExtraFormats(const QTextBlock &block,
     if (block.layout() == nullptr || blockLength == 0)
         return;
 
+    const QString preeditText = block.layout()->preeditAreaText();
+    if (!preeditText.isEmpty()) {
+        QVector<QTextLayout::FormatRange> additionalRanges;
+        const int preeditPosition = block.layout()->preeditAreaPosition();
+        for (QTextLayout::FormatRange &r : formats) {
+            if (r.start >= preeditPosition) {
+                r.start += preeditText.length();
+            } else if (r.start + r.length > preeditPosition) {
+                QTextLayout::FormatRange afterPreeditRange = r;
+                afterPreeditRange.start = preeditPosition + preeditText.length();
+                afterPreeditRange.length = r.length - (preeditPosition - r.start);
+                additionalRanges << afterPreeditRange;
+                r.length = preeditPosition - r.start;
+            }
+        }
+        formats << additionalRanges;
+    }
+
     Utils::sort(formats, byStartOfRange);
 
     const QVector<QTextLayout::FormatRange> all = block.layout()->formats();
@@ -672,11 +701,11 @@ void SyntaxHighlighter::setExtraFormats(const QTextBlock &block,
     QVector<QTextLayout::FormatRange> formatsToApply;
     std::tie(previousSemanticFormats, formatsToApply)
         = Utils::partition(all, [](const QTextLayout::FormatRange &r) {
-              return r.format.hasProperty(QTextFormat::UserProperty);
+              return r.format.property(SemanticHighlight).toBool();
           });
 
     for (auto &format : formats)
-        format.format.setProperty(QTextFormat::UserProperty, true);
+        format.format.setProperty(SemanticHighlight, true);
 
     if (formats.size() == previousSemanticFormats.size()) {
         Utils::sort(previousSemanticFormats, byStartOfRange);
@@ -703,7 +732,7 @@ void SyntaxHighlighter::clearExtraFormats(const QTextBlock &block)
 
     const QVector<QTextLayout::FormatRange> formatsToApply
         = Utils::filtered(block.layout()->formats(), [](const QTextLayout::FormatRange &r) {
-              return !r.format.hasProperty(QTextFormat::UserProperty);
+              return !r.format.property(SemanticHighlight).toBool();
           });
 
     bool wasInReformatBlocks = d->inReformatBlocks;
@@ -768,7 +797,7 @@ FontSettings SyntaxHighlighter::fontSettings() const
     return d->fontSettings;
 }
 /*!
-    The syntax highlighter is not anymore reacting to the text document if \a noAutmatic is
+    The syntax highlighter is not anymore reacting to the text document if \a noAutomatic is
     \c true.
 */
 void SyntaxHighlighter::setNoAutomaticHighlighting(bool noAutomatic)
@@ -831,6 +860,21 @@ QTextCharFormat SyntaxHighlighter::formatForCategory(int category) const
     return d->formats.at(category);
 }
 
+QTextCharFormat SyntaxHighlighter::whitespacified(const QTextCharFormat &fmt)
+{
+    Q_D(SyntaxHighlighter);
+    QTextCharFormat format = d->whitespaceFormat;
+    format.setBackground(fmt.background());
+    return format;
+}
+
+QTextCharFormat SyntaxHighlighter::asSyntaxHighlight(const QTextCharFormat &fmt)
+{
+    QTextCharFormat format = fmt;
+    format.setProperty(SyntaxHighlight, true);
+    return format;
+}
+
 void SyntaxHighlighter::highlightBlock(const QString &text)
 {
     formatSpaces(text);
@@ -841,7 +885,7 @@ void SyntaxHighlighterPrivate::updateFormats(const FontSettings &fontSettings)
     this->fontSettings = fontSettings;
     // C_TEXT is handled by text editor's foreground and background color,
     // so use empty format for that
-    for (const auto &pair : qAsConst(formatCategories)) {
+    for (const auto &pair : std::as_const(formatCategories)) {
         formats[pair.first] = pair.second == C_TEXT ? QTextCharFormat()
                                                     : fontSettings.toTextCharFormat(pair.second);
     }
